# -*- coding: utf-8 -*-
"""Cisco DNA Center API wrappers.

Copyright (c) 2019-2021 Cisco Systems.

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""


from dnacentersdk.config import (
    DEFAULT_DEBUG,
    DEFAULT_VERSION,
    DEFAULT_BASE_URL,
    DEFAULT_SINGLE_REQUEST_TIMEOUT,
    DEFAULT_WAIT_ON_RATE_LIMIT,
    DEFAULT_VERIFY,
    DEFAULT_VERIFY_USER_AGENT,
)
import dnacentersdk.environment as dnacenter_environment
from dnacentersdk.exceptions import AccessTokenError, VersionError
from dnacentersdk.models.mydict import mydict_data_factory
from dnacentersdk.models.schema_validator import SchemaValidator
from dnacentersdk.restsession import RestSession
from dnacentersdk.utils import check_type

from .authentication import Authentication

from .v2_2_2_3.application_policy import \
    ApplicationPolicy as ApplicationPolicy_v2_2_2_3
from .v2_2_2_3.applications import \
    Applications as Applications_v2_2_2_3
from .v2_2_2_3.authentication_management import \
    AuthenticationManagement as AuthenticationManagement_v2_2_2_3
from .v2_2_2_3.clients import \
    Clients as Clients_v2_2_2_3
from .v2_2_2_3.command_runner import \
    CommandRunner as CommandRunner_v2_2_2_3
from .v2_2_2_3.compliance import \
    Compliance as Compliance_v2_2_2_3
from .v2_2_2_3.configuration_archive import \
    ConfigurationArchive as ConfigurationArchive_v2_2_2_3
from .v2_2_2_3.configuration_templates import \
    ConfigurationTemplates as ConfigurationTemplates_v2_2_2_3
from .v2_2_2_3.device_onboarding_pnp import \
    DeviceOnboardingPnp as DeviceOnboardingPnp_v2_2_2_3
from .v2_2_2_3.device_replacement import \
    DeviceReplacement as DeviceReplacement_v2_2_2_3
from .v2_2_2_3.devices import \
    Devices as Devices_v2_2_2_3
from .v2_2_2_3.discovery import \
    Discovery as Discovery_v2_2_2_3
from .v2_2_2_3.event_management import \
    EventManagement as EventManagement_v2_2_2_3
from .v2_2_2_3.file import \
    File as File_v2_2_2_3
from .v2_2_2_3.health_and_performance import \
    HealthAndPerformance as HealthAndPerformance_v2_2_2_3
from .v2_2_2_3.itsm import \
    Itsm as Itsm_v2_2_2_3
from .v2_2_2_3.issues import \
    Issues as Issues_v2_2_2_3
from .v2_2_2_3.licenses import \
    Licenses as Licenses_v2_2_2_3
from .v2_2_2_3.network_settings import \
    NetworkSettings as NetworkSettings_v2_2_2_3
from .v2_2_2_3.path_trace import \
    PathTrace as PathTrace_v2_2_2_3
from .v2_2_2_3.reports import \
    Reports as Reports_v2_2_2_3
from .v2_2_2_3.sda import \
    Sda as Sda_v2_2_2_3
from .v2_2_2_3.security_advisories import \
    SecurityAdvisories as SecurityAdvisories_v2_2_2_3
from .v2_2_2_3.platform_configuration import \
    PlatformConfiguration as PlatformConfiguration_v2_2_2_3
from .v2_2_2_3.sensors import \
    Sensors as Sensors_v2_2_2_3
from .v2_2_2_3.site_design import \
    SiteDesign as SiteDesign_v2_2_2_3
from .v2_2_2_3.sites import \
    Sites as Sites_v2_2_2_3
from .v2_2_2_3.software_image_management_swim import \
    SoftwareImageManagementSwim as SoftwareImageManagementSwim_v2_2_2_3
from .v2_2_2_3.tag import \
    Tag as Tag_v2_2_2_3
from .v2_2_2_3.task import \
    Task as Task_v2_2_2_3
from .v2_2_2_3.topology import \
    Topology as Topology_v2_2_2_3
from .v2_2_2_3.users import \
    Users as Users_v2_2_2_3
from .v2_2_2_3.wireless import \
    Wireless as Wireless_v2_2_2_3
from .v2_2_3_3.application_policy import \
    ApplicationPolicy as ApplicationPolicy_v2_2_3_3
from .v2_2_3_3.applications import \
    Applications as Applications_v2_2_3_3
from .v2_2_3_3.platform_configuration import \
    PlatformConfiguration as PlatformConfiguration_v2_2_3_3
from .v2_2_3_3.authentication_management import \
    AuthenticationManagement as AuthenticationManagement_v2_2_3_3
from .v2_2_3_3.clients import \
    Clients as Clients_v2_2_3_3
from .v2_2_3_3.command_runner import \
    CommandRunner as CommandRunner_v2_2_3_3
from .v2_2_3_3.compliance import \
    Compliance as Compliance_v2_2_3_3
from .v2_2_3_3.configuration_archive import \
    ConfigurationArchive as ConfigurationArchive_v2_2_3_3
from .v2_2_3_3.configuration_templates import \
    ConfigurationTemplates as ConfigurationTemplates_v2_2_3_3
from .v2_2_3_3.device_onboarding_pnp import \
    DeviceOnboardingPnp as DeviceOnboardingPnp_v2_2_3_3
from .v2_2_3_3.device_replacement import \
    DeviceReplacement as DeviceReplacement_v2_2_3_3
from .v2_2_3_3.devices import \
    Devices as Devices_v2_2_3_3
from .v2_2_3_3.disaster_recovery import \
    DisasterRecovery as DisasterRecovery_v2_2_3_3
from .v2_2_3_3.discovery import \
    Discovery as Discovery_v2_2_3_3
from .v2_2_3_3.event_management import \
    EventManagement as EventManagement_v2_2_3_3
from .v2_2_3_3.fabric_wireless import \
    FabricWireless as FabricWireless_v2_2_3_3
from .v2_2_3_3.file import \
    File as File_v2_2_3_3
from .v2_2_3_3.health_and_performance import \
    HealthAndPerformance as HealthAndPerformance_v2_2_3_3
from .v2_2_3_3.itsm import \
    Itsm as Itsm_v2_2_3_3
from .v2_2_3_3.issues import \
    Issues as Issues_v2_2_3_3
from .v2_2_3_3.licenses import \
    Licenses as Licenses_v2_2_3_3
from .v2_2_3_3.network_settings import \
    NetworkSettings as NetworkSettings_v2_2_3_3
from .v2_2_3_3.path_trace import \
    PathTrace as PathTrace_v2_2_3_3
from .v2_2_3_3.policy import \
    Policy as Policy_v2_2_3_3
from .v2_2_3_3.reports import \
    Reports as Reports_v2_2_3_3
from .v2_2_3_3.sda import \
    Sda as Sda_v2_2_3_3
from .v2_2_3_3.security_advisories import \
    SecurityAdvisories as SecurityAdvisories_v2_2_3_3
from .v2_2_3_3.sensors import \
    Sensors as Sensors_v2_2_3_3
from .v2_2_3_3.site_design import \
    SiteDesign as SiteDesign_v2_2_3_3
from .v2_2_3_3.sites import \
    Sites as Sites_v2_2_3_3
from .v2_2_3_3.software_image_management_swim import \
    SoftwareImageManagementSwim as SoftwareImageManagementSwim_v2_2_3_3
from .v2_2_3_3.tag import \
    Tag as Tag_v2_2_3_3
from .v2_2_3_3.task import \
    Task as Task_v2_2_3_3
from .v2_2_3_3.topology import \
    Topology as Topology_v2_2_3_3
from .v2_2_3_3.users import \
    Users as Users_v2_2_3_3
from .v2_2_3_3.wireless import \
    Wireless as Wireless_v2_2_3_3
from .v2_3_3_0.application_policy import \
    ApplicationPolicy as ApplicationPolicy_v2_3_3_0
from .v2_3_3_0.applications import \
    Applications as Applications_v2_3_3_0
from .v2_3_3_0.cisco_dna_center_system import \
    CiscoDnaCenterSystem as CiscoDnaCenterSystem_v2_3_3_0
from .v2_3_3_0.clients import \
    Clients as Clients_v2_3_3_0
from .v2_3_3_0.command_runner import \
    CommandRunner as CommandRunner_v2_3_3_0
from .v2_3_3_0.compliance import \
    Compliance as Compliance_v2_3_3_0
from .v2_3_3_0.configuration_archive import \
    ConfigurationArchive as ConfigurationArchive_v2_3_3_0
from .v2_3_3_0.configuration_templates import \
    ConfigurationTemplates as ConfigurationTemplates_v2_3_3_0
from .v2_3_3_0.device_onboarding_pnp import \
    DeviceOnboardingPnp as DeviceOnboardingPnp_v2_3_3_0
from .v2_3_3_0.device_replacement import \
    DeviceReplacement as DeviceReplacement_v2_3_3_0
from .v2_3_3_0.devices import \
    Devices as Devices_v2_3_3_0
from .v2_3_3_0.platform_configuration import \
    PlatformConfiguration as PlatformConfiguration_v2_3_3_0
from .v2_3_3_0.discovery import \
    Discovery as Discovery_v2_3_3_0
from .v2_3_3_0.event_management import \
    EventManagement as EventManagement_v2_3_3_0
from .v2_3_3_0.fabric_wireless import \
    FabricWireless as FabricWireless_v2_3_3_0
from .v2_3_3_0.file import \
    File as File_v2_3_3_0
from .v2_3_3_0.health_and_performance import \
    HealthAndPerformance as HealthAndPerformance_v2_3_3_0
from .v2_3_3_0.itsm import \
    Itsm as Itsm_v2_3_3_0
from .v2_3_3_0.issues import \
    Issues as Issues_v2_3_3_0
from .v2_3_3_0.lan_automation import \
    LanAutomation as LanAutomation_v2_3_3_0
from .v2_3_3_0.licenses import \
    Licenses as Licenses_v2_3_3_0
from .v2_3_3_0.network_settings import \
    NetworkSettings as NetworkSettings_v2_3_3_0
from .v2_3_3_0.path_trace import \
    PathTrace as PathTrace_v2_3_3_0
from .v2_3_3_0.reports import \
    Reports as Reports_v2_3_3_0
from .v2_3_3_0.sda import \
    Sda as Sda_v2_3_3_0
from .v2_3_3_0.security_advisories import \
    SecurityAdvisories as SecurityAdvisories_v2_3_3_0
from .v2_3_3_0.sensors import \
    Sensors as Sensors_v2_3_3_0
from .v2_3_3_0.site_design import \
    SiteDesign as SiteDesign_v2_3_3_0
from .v2_3_3_0.sites import \
    Sites as Sites_v2_3_3_0
from .v2_3_3_0.software_image_management_swim import \
    SoftwareImageManagementSwim as SoftwareImageManagementSwim_v2_3_3_0
from .v2_3_3_0.system_settings import \
    SystemSettings as SystemSettings_v2_3_3_0
from .v2_3_3_0.tag import \
    Tag as Tag_v2_3_3_0
from .v2_3_3_0.task import \
    Task as Task_v2_3_3_0
from .v2_3_3_0.topology import \
    Topology as Topology_v2_3_3_0
from .v2_3_3_0.users import \
    Users as Users_v2_3_3_0
from .v2_3_3_0.wireless import \
    Wireless as Wireless_v2_3_3_0
from .v2_3_5_3.application_policy import \
    ApplicationPolicy as ApplicationPolicy_v2_3_5_3
from .v2_3_5_3.applications import \
    Applications as Applications_v2_3_5_3
from .v2_3_5_3.authentication_management import \
    AuthenticationManagement as AuthenticationManagement_v2_3_5_3
from .v2_3_5_3.cisco_dna_center_system import \
    CiscoDnaCenterSystem as CiscoDnaCenterSystem_v2_3_5_3
from .v2_3_5_3.clients import \
    Clients as Clients_v2_3_5_3
from .v2_3_5_3.command_runner import \
    CommandRunner as CommandRunner_v2_3_5_3
from .v2_3_5_3.compliance import \
    Compliance as Compliance_v2_3_5_3
from .v2_3_5_3.configuration_archive import \
    ConfigurationArchive as ConfigurationArchive_v2_3_5_3
from .v2_3_5_3.configuration_templates import \
    ConfigurationTemplates as ConfigurationTemplates_v2_3_5_3
from .v2_3_5_3.device_onboarding_pnp import \
    DeviceOnboardingPnp as DeviceOnboardingPnp_v2_3_5_3
from .v2_3_5_3.device_replacement import \
    DeviceReplacement as DeviceReplacement_v2_3_5_3
from .v2_3_5_3.devices import \
    Devices as Devices_v2_3_5_3
from .v2_3_5_3.discovery import \
    Discovery as Discovery_v2_3_5_3
from .v2_3_5_3.eox import \
    Eox as Eox_v2_3_5_3
from .v2_3_5_3.event_management import \
    EventManagement as EventManagement_v2_3_5_3
from .v2_3_5_3.fabric_wireless import \
    FabricWireless as FabricWireless_v2_3_5_3
from .v2_3_5_3.file import \
    File as File_v2_3_5_3
from .v2_3_5_3.health_and_performance import \
    HealthAndPerformance as HealthAndPerformance_v2_3_5_3
from .v2_3_5_3.itsm import \
    Itsm as Itsm_v2_3_5_3
from .v2_3_5_3.itsm_integration import \
    ItsmIntegration as ItsmIntegration_v2_3_5_3
from .v2_3_5_3.issues import \
    Issues as Issues_v2_3_5_3
from .v2_3_5_3.lan_automation import \
    LanAutomation as LanAutomation_v2_3_5_3
from .v2_3_5_3.licenses import \
    Licenses as Licenses_v2_3_5_3
from .v2_3_5_3.network_settings import \
    NetworkSettings as NetworkSettings_v2_3_5_3
from .v2_3_5_3.path_trace import \
    PathTrace as PathTrace_v2_3_5_3
from .v2_3_5_3.platform import \
    Platform as Platform_v2_3_5_3
from .v2_3_5_3.reports import \
    Reports as Reports_v2_3_5_3
from .v2_3_5_3.sda import \
    Sda as Sda_v2_3_5_3
from .v2_3_5_3.security_advisories import \
    SecurityAdvisories as SecurityAdvisories_v2_3_5_3
from .v2_3_5_3.sensors import \
    Sensors as Sensors_v2_3_5_3
from .v2_3_5_3.site_design import \
    SiteDesign as SiteDesign_v2_3_5_3
from .v2_3_5_3.sites import \
    Sites as Sites_v2_3_5_3
from .v2_3_5_3.software_image_management_swim import \
    SoftwareImageManagementSwim as SoftwareImageManagementSwim_v2_3_5_3
from .v2_3_5_3.system_settings import \
    SystemSettings as SystemSettings_v2_3_5_3
from .v2_3_5_3.tag import \
    Tag as Tag_v2_3_5_3
from .v2_3_5_3.task import \
    Task as Task_v2_3_5_3
from .v2_3_5_3.topology import \
    Topology as Topology_v2_3_5_3
from .v2_3_5_3.user_and_roles import \
    UserandRoles as UserandRoles_v2_3_5_3
from .v2_3_5_3.users import \
    Users as Users_v2_3_5_3
from .v2_3_5_3.wireless import \
    Wireless as Wireless_v2_3_5_3

from .v2_3_7_6.ai_endpoint_analytics import \
    AIEndpointAnalytics as AIEndpointAnalytics_v2_3_7_6
from .v2_3_7_6.application_policy import \
    ApplicationPolicy as ApplicationPolicy_v2_3_7_6
from .v2_3_7_6.applications import \
    Applications as Applications_v2_3_7_6
from .v2_3_7_6.cisco_trusted_certificates import \
    CiscoTrustedCertificates as CiscoTrustedCertificates_v2_3_7_6
from .v2_3_7_6.clients import \
    Clients as Clients_v2_3_7_6
from .v2_3_7_6.command_runner import \
    CommandRunner as CommandRunner_v2_3_7_6
from .v2_3_7_6.compliance import \
    Compliance as Compliance_v2_3_7_6
from .v2_3_7_6.configuration_archive import \
    ConfigurationArchive as ConfigurationArchive_v2_3_7_6
from .v2_3_7_6.configuration_templates import \
    ConfigurationTemplates as ConfigurationTemplates_v2_3_7_6
from .v2_3_7_6.device_onboarding_pnp import \
    DeviceOnboardingPnp as DeviceOnboardingPnp_v2_3_7_6
from .v2_3_7_6.device_replacement import \
    DeviceReplacement as DeviceReplacement_v2_3_7_6
from .v2_3_7_6.devices import \
    Devices as Devices_v2_3_7_6
from .v2_3_7_6.authentication_management import \
    AuthenticationManagement as AuthenticationManagement_v2_3_7_6
from .v2_3_7_6.disaster_recovery import \
    DisasterRecovery as DisasterRecovery_v2_3_7_6
from .v2_3_7_6.discovery import \
    Discovery as Discovery_v2_3_7_6
from .v2_3_7_6.eox import \
    EoX as EoX_v2_3_7_6
from .v2_3_7_6.event_management import \
    EventManagement as EventManagement_v2_3_7_6
from .v2_3_7_6.fabric_wireless import \
    FabricWireless as FabricWireless_v2_3_7_6
from .v2_3_7_6.file import \
    File as File_v2_3_7_6
from .v2_3_7_6.health_and_performance import \
    HealthAndPerformance as HealthAndPerformance_v2_3_7_6
from .v2_3_7_6.itsm import \
    Itsm as Itsm_v2_3_7_6
from .v2_3_7_6.itsm_integration import \
    ItsmIntegration as ItsmIntegration_v2_3_7_6
from .v2_3_7_6.issues import \
    Issues as Issues_v2_3_7_6
from .v2_3_7_6.lan_automation import \
    LanAutomation as LanAutomation_v2_3_7_6
from .v2_3_7_6.licenses import \
    Licenses as Licenses_v2_3_7_6
from .v2_3_7_6.network_settings import \
    NetworkSettings as NetworkSettings_v2_3_7_6
from .v2_3_7_6.path_trace import \
    PathTrace as PathTrace_v2_3_7_6
from .v2_3_7_6.platform import \
    Platform as Platform_v2_3_7_6
from .v2_3_7_6.reports import \
    Reports as Reports_v2_3_7_6
from .v2_3_7_6.sda import \
    Sda as Sda_v2_3_7_6
from .v2_3_7_6.security_advisories import \
    SecurityAdvisories as SecurityAdvisories_v2_3_7_6
from .v2_3_7_6.sensors import \
    Sensors as Sensors_v2_3_7_6
from .v2_3_7_6.site_design import \
    SiteDesign as SiteDesign_v2_3_7_6
from .v2_3_7_6.sites import \
    Sites as Sites_v2_3_7_6
from .v2_3_7_6.software_image_management_swim import \
    SoftwareImageManagementSwim as SoftwareImageManagementSwim_v2_3_7_6
from .v2_3_7_6.system_settings import \
    SystemSettings as SystemSettings_v2_3_7_6
from .v2_3_7_6.tag import \
    Tag as Tag_v2_3_7_6
from .v2_3_7_6.task import \
    Task as Task_v2_3_7_6
from .v2_3_7_6.topology import \
    Topology as Topology_v2_3_7_6
from .v2_3_7_6.user_and_roles import \
    UserandRoles as UserandRoles_v2_3_7_6
from .v2_3_7_6.users import \
    Users as Users_v2_3_7_6
from .v2_3_7_6.wireless import \
    Wireless as Wireless_v2_3_7_6
from .v2_3_7_9.ai_endpoint_analytics import \
    AIEndpointAnalytics as AIEndpointAnalytics_v2_3_7_9
from .v2_3_7_9.application_policy import \
    ApplicationPolicy as ApplicationPolicy_v2_3_7_9
from .v2_3_7_9.applications import \
    Applications as Applications_v2_3_7_9
from .v2_3_7_9.authentication_management import \
    AuthenticationManagement as AuthenticationManagement_v2_3_7_9
from .v2_3_7_9.cisco_i_m_c import \
    CiscoIMC as CiscoIMC_v2_3_7_9
from .v2_3_7_9.cisco_trusted_certificates import \
    CiscoTrustedCertificates as CiscoTrustedCertificates_v2_3_7_9
from .v2_3_7_9.clients import \
    Clients as Clients_v2_3_7_9
from .v2_3_7_9.command_runner import \
    CommandRunner as CommandRunner_v2_3_7_9
from .v2_3_7_9.compliance import \
    Compliance as Compliance_v2_3_7_9
from .v2_3_7_9.configuration_archive import \
    ConfigurationArchive as ConfigurationArchive_v2_3_7_9
from .v2_3_7_9.configuration_templates import \
    ConfigurationTemplates as ConfigurationTemplates_v2_3_7_9
from .v2_3_7_9.device_onboarding_pnp import \
    DeviceOnboardingPnp as DeviceOnboardingPnp_v2_3_7_9
from .v2_3_7_9.device_replacement import \
    DeviceReplacement as DeviceReplacement_v2_3_7_9
from .v2_3_7_9.devices import \
    Devices as Devices_v2_3_7_9
from .v2_3_7_9.disaster_recovery import \
    DisasterRecovery as DisasterRecovery_v2_3_7_9
from .v2_3_7_9.discovery import \
    Discovery as Discovery_v2_3_7_9
from .v2_3_7_9.eox import \
    EoX as EoX_v2_3_7_9
from .v2_3_7_9.event_management import \
    EventManagement as EventManagement_v2_3_7_9
from .v2_3_7_9.fabric_wireless import \
    FabricWireless as FabricWireless_v2_3_7_9
from .v2_3_7_9.file import \
    File as File_v2_3_7_9
from .v2_3_7_9.health_and_performance import \
    HealthAndPerformance as HealthAndPerformance_v2_3_7_9
from .v2_3_7_9.itsm import \
    Itsm as Itsm_v2_3_7_9
from .v2_3_7_9.itsm_integration import \
    ItsmIntegration as ItsmIntegration_v2_3_7_9
from .v2_3_7_9.issues import \
    Issues as Issues_v2_3_7_9
from .v2_3_7_9.lan_automation import \
    LanAutomation as LanAutomation_v2_3_7_9
from .v2_3_7_9.licenses import \
    Licenses as Licenses_v2_3_7_9
from .v2_3_7_9.network_settings import \
    NetworkSettings as NetworkSettings_v2_3_7_9
from .v2_3_7_9.path_trace import \
    PathTrace as PathTrace_v2_3_7_9
from .v2_3_7_9.platform import \
    Platform as Platform_v2_3_7_9
from .v2_3_7_9.reports import \
    Reports as Reports_v2_3_7_9
from .v2_3_7_9.sda import \
    Sda as Sda_v2_3_7_9
from .v2_3_7_9.security_advisories import \
    SecurityAdvisories as SecurityAdvisories_v2_3_7_9
from .v2_3_7_9.sensors import \
    Sensors as Sensors_v2_3_7_9
from .v2_3_7_9.site_design import \
    SiteDesign as SiteDesign_v2_3_7_9
from .v2_3_7_9.sites import \
    Sites as Sites_v2_3_7_9
from .v2_3_7_9.software_image_management_swim import \
    SoftwareImageManagementSwim as SoftwareImageManagementSwim_v2_3_7_9
from .v2_3_7_9.system_settings import \
    SystemSettings as SystemSettings_v2_3_7_9
from .v2_3_7_9.tag import \
    Tag as Tag_v2_3_7_9
from .v2_3_7_9.task import \
    Task as Task_v2_3_7_9
from .v2_3_7_9.topology import \
    Topology as Topology_v2_3_7_9
from .v2_3_7_9.user_and_roles import \
    UserandRoles as UserandRoles_v2_3_7_9
from .v2_3_7_9.users import \
    Users as Users_v2_3_7_9
from .v2_3_7_9.wireless import \
    Wireless as Wireless_v2_3_7_9
from .custom_caller import CustomCaller


class DNACenterAPI(object):
    """Cisco DNA Center API wrapper.

    Creates a 'session' for all API calls through a created DNACenterAPI
    object.  The 'session' handles authentication, provides the needed headers,
    and checks all responses for error conditions.

    DNACenterAPI wraps all of the individual DNA Center APIs and represents
    them in a simple hierarchical structure.
    """

    def __init__(self, username=None,
                 password=None,
                 encoded_auth=None,
                 base_url=None,
                 single_request_timeout=None,
                 wait_on_rate_limit=None,
                 session=None,
                 verify=None,
                 version=None,
                 debug=None,
                 object_factory=mydict_data_factory,
                 validator=SchemaValidator,
                 user_agent =None):
        """Create a new DNACenterAPI object.
        An access token is required to interact with the DNA Center APIs.
        This package supports two methods for you to generate the
        authorization token:

          1. Provide a encoded_auth value (username:password encoded in
          base 64). *This has priority over the following method*

          2. Provide username and password values.

        This package supports two methods for you to set those values:

          1. Provide the parameter. That is the encoded_auth or
          username and password parameters.

          2. If an argument is not supplied, the package checks for
          its environment variable counterpart. That is the
          DNA_CENTER_ENCODED_AUTH, DNA_CENTER_USERNAME,
          DNA_CENTER_PASSWORD.

        When not given enough parameters an AccessTokenError is raised.

        Args:
            base_url(str): The base URL to be prefixed to the
                individual API endpoint suffixes.
                Defaults to the DNA_CENTER_BASE_URL environment variable or
                dnacentersdk.config.DEFAULT_BASE_URL
                if the environment variable is not set.
            username(str): HTTP Basic Auth username.
            password(str): HTTP Basic Auth password.
            encoded_auth(str): HTTP Basic Auth base64 encoded string.
            single_request_timeout(int): Timeout (in seconds) for RESTful HTTP
                requests. Defaults to the DNA_CENTER_SINGLE_REQUEST_TIMEOUT
                environment variable or
                dnacentersdk.config.DEFAULT_SINGLE_REQUEST_TIMEOUT
                if the environment variable is not set.
            wait_on_rate_limit(bool): Enables or disables automatic rate-limit
                handling. Defaults to the DNA_CENTER_WAIT_ON_RATE_LIMIT
                environment variable or
                dnacentersdk.config.DEFAULT_WAIT_ON_RATE_LIMIT
                if the environment variable is not set.
            verify(bool,str): Controls whether we verify the server's
                TLS certificate, or a string, in which case it must be a path
                to a CA bundle to use. Defaults to the DNA_CENTER_VERIFY
                (or DNA_CENTER_VERIFY_STRING) environment variable or
                dnacentersdk.config.DEFAULT_VERIFY if the environment
                variables are not set.
            session(requests.Session): Optionally inject a `requests.Session`
                instance to use for HTTP operations.
            version(str): Controls which version of DNA_CENTER to use.
                Defaults to the DNA_CENTER_VERSION environment variable or
                dnacentersdk.config.DEFAULT_VERSION
                if the environment variable is not set.
            debug(bool,str): Controls whether to log information about
                DNA Center APIs' request and response process.
                Defaults to the DNA_CENTER_DEBUG environment variable or False
                if the environment variable is not set.
            object_factory(callable): The factory function to use to create
                Python objects from the returned DNA Center JSON data objects.
            validator(callable): The factory function to use to validate
                Python objects sent in the body of the request.

        Returns:
            DNACenterAPI: A new DNACenterAPI object.

        Raises:
            TypeError: If the parameter types are incorrect.
            AccessTokenError: If an access token is not provided via the
                access_token argument or an environment variable.
            VersionError: If the version is not provided via the version
                argument or an environment variable, or it is not a
                DNA Center API supported version
                [  '2.2.2.3', '2.2.3.3',
                '2.3.3.0', '2.3.5.3', '2.3.7.6'].

        """
        username = username or dnacenter_environment.get_env_username()
        password = password or dnacenter_environment.get_env_password()
        encoded_auth = encoded_auth or dnacenter_environment.get_env_encoded_auth()
        base_url = base_url or dnacenter_environment.get_env_base_url() or DEFAULT_BASE_URL
        user_agent = user_agent or dnacenter_environment.get_env_user_agent()

        if single_request_timeout is None:
            single_request_timeout = dnacenter_environment.get_env_single_request_timeout() or DEFAULT_SINGLE_REQUEST_TIMEOUT

        if wait_on_rate_limit is None:
            wait_on_rate_limit = dnacenter_environment.get_env_wait_on_rate_limit() or DEFAULT_WAIT_ON_RATE_LIMIT

        if verify is None:
            verify = dnacenter_environment.get_env_verify()
            if verify is None:
                verify = DEFAULT_VERIFY

        version = version or dnacenter_environment.get_env_version() or DEFAULT_VERSION

        if debug is None:
            debug = dnacenter_environment.get_env_debug() or DEFAULT_DEBUG

        if user_agent is None:
            user_agent = dnacenter_environment.get_env_user_agent() or DEFAULT_VERIFY_USER_AGENT

        check_type(base_url, str)
        check_type(single_request_timeout, int)
        check_type(wait_on_rate_limit, bool)
        check_type(debug, (bool, str), may_be_none=True)
        check_type(username, str, may_be_none=True)
        check_type(password, str, may_be_none=True)
        check_type(encoded_auth, str, may_be_none=True)
        check_type(verify, (bool, str), may_be_none=False)
        check_type(version, str, may_be_none=False)
        check_type(user_agent, str, may_be_none=False)

        if version not in ['2.2.2.3', '2.2.3.3',
                           '2.3.3.0', '2.3.5.3',
                           '2.3.7.6','2.3.7.9']:
            raise VersionError(
                'Unknown API version, '
                + 'known versions are {}'.format(
                    '  2.2.2.3, 2.2.3.3, 2.3.3.0, 2.3.5.3, 2.3.7.6 and '
                    + '2.3.7.9'
                )
            )

        if isinstance(debug, str):
            debug = 'true' in debug.lower()

        # Init Authentication wrapper early to use for basicAuth requests
        self.authentication = Authentication(
            base_url, object_factory,
            single_request_timeout=single_request_timeout,
            verify=verify,
        )

        # Check if the user has provided the required basicAuth parameters
        if encoded_auth is None and (username is None or password is None):
            raise AccessTokenError(
                "You need an access token to interact with the DNA Center"
                " APIs. DNA Center uses HTTP Basic Auth to create an access"
                " token. You must provide the username and password or just"
                " the encoded_auth, either by setting each parameter or its"
                " environment variable counterpart ("
                "DNA_CENTER_USERNAME, DNA_CENTER_PASSWORD,"
                " DNA_CENTER_ENCODED_AUTH)."
            )

        def get_access_token():
            return self.authentication.authentication_api(
                username=username,
                password=password,
                encoded_auth=encoded_auth).Token

        # Create the API session
        # All of the API calls associated with a DNACenterAPI object will
        # leverage a single RESTful 'session' connecting to the DNA Center
        # cloud.
        self._session = RestSession(
            get_access_token=get_access_token,
            access_token=get_access_token(),
            base_url=base_url,
            single_request_timeout=single_request_timeout,
            wait_on_rate_limit=wait_on_rate_limit,
            session=session,
            verify=verify,
            version=version,
            debug=debug,
            user_agent = user_agent,
        )

        _validator = validator(version).json_schema_validate

        # API wrappers
        if version == '2.2.2.3':
            self.application_policy = \
                ApplicationPolicy_v2_2_2_3(
                    self._session, object_factory, _validator
                )
            self.applications = \
                Applications_v2_2_2_3(
                    self._session, object_factory, _validator
                )
            self.authentication_management = \
                AuthenticationManagement_v2_2_2_3(
                    self._session, object_factory, _validator
                )
            self.clients = \
                Clients_v2_2_2_3(
                    self._session, object_factory, _validator
                )
            self.command_runner = \
                CommandRunner_v2_2_2_3(
                    self._session, object_factory, _validator
                )
            self.compliance = \
                Compliance_v2_2_2_3(
                    self._session, object_factory, _validator
                )
            self.configuration_archive = \
                ConfigurationArchive_v2_2_2_3(
                    self._session, object_factory, _validator
                )
            self.configuration_templates = \
                ConfigurationTemplates_v2_2_2_3(
                    self._session, object_factory, _validator
                )
            self.device_onboarding_pnp = \
                DeviceOnboardingPnp_v2_2_2_3(
                    self._session, object_factory, _validator
                )
            self.device_replacement = \
                DeviceReplacement_v2_2_2_3(
                    self._session, object_factory, _validator
                )
            self.devices = \
                Devices_v2_2_2_3(
                    self._session, object_factory, _validator
                )
            self.discovery = \
                Discovery_v2_2_2_3(
                    self._session, object_factory, _validator
                )
            self.event_management = \
                EventManagement_v2_2_2_3(
                    self._session, object_factory, _validator
                )
            self.file = \
                File_v2_2_2_3(
                    self._session, object_factory, _validator
                )
            self.health_and_performance = \
                HealthAndPerformance_v2_2_2_3(
                    self._session, object_factory, _validator
                )
            self.itsm = \
                Itsm_v2_2_2_3(
                    self._session, object_factory, _validator
                )
            self.issues = \
                Issues_v2_2_2_3(
                    self._session, object_factory, _validator
                )
            self.licenses = \
                Licenses_v2_2_2_3(
                    self._session, object_factory, _validator
                )
            self.network_settings = \
                NetworkSettings_v2_2_2_3(
                    self._session, object_factory, _validator
                )
            self.path_trace = \
                PathTrace_v2_2_2_3(
                    self._session, object_factory, _validator
                )
            self.platform_configuration = \
                PlatformConfiguration_v2_2_2_3(
                    self._session, object_factory, _validator
                )
            self.reports = \
                Reports_v2_2_2_3(
                    self._session, object_factory, _validator
                )
            self.sda = \
                Sda_v2_2_2_3(
                    self._session, object_factory, _validator
                )
            self.security_advisories = \
                SecurityAdvisories_v2_2_2_3(
                    self._session, object_factory, _validator
                )
            self.sensors = \
                Sensors_v2_2_2_3(
                    self._session, object_factory, _validator
                )
            self.site_design = \
                SiteDesign_v2_2_2_3(
                    self._session, object_factory, _validator
                )
            self.sites = \
                Sites_v2_2_2_3(
                    self._session, object_factory, _validator
                )
            self.software_image_management_swim = \
                SoftwareImageManagementSwim_v2_2_2_3(
                    self._session, object_factory, _validator
                )
            self.tag = \
                Tag_v2_2_2_3(
                    self._session, object_factory, _validator
                )
            self.task = \
                Task_v2_2_2_3(
                    self._session, object_factory, _validator
                )
            self.topology = \
                Topology_v2_2_2_3(
                    self._session, object_factory, _validator
                )
            self.users = \
                Users_v2_2_2_3(
                    self._session, object_factory, _validator
                )
            self.wireless = \
                Wireless_v2_2_2_3(
                    self._session, object_factory, _validator
                )
        if version == '2.2.3.3':
            self.application_policy = \
                ApplicationPolicy_v2_2_3_3(
                    self._session, object_factory, _validator
                )
            self.applications = \
                Applications_v2_2_3_3(
                    self._session, object_factory, _validator
                )
            self.authentication_management = \
                AuthenticationManagement_v2_2_3_3(
                    self._session, object_factory, _validator
                )
            self.clients = \
                Clients_v2_2_3_3(
                    self._session, object_factory, _validator
                )
            self.command_runner = \
                CommandRunner_v2_2_3_3(
                    self._session, object_factory, _validator
                )
            self.compliance = \
                Compliance_v2_2_3_3(
                    self._session, object_factory, _validator
                )
            self.configuration_archive = \
                ConfigurationArchive_v2_2_3_3(
                    self._session, object_factory, _validator
                )
            self.configuration_templates = \
                ConfigurationTemplates_v2_2_3_3(
                    self._session, object_factory, _validator
                )
            self.device_onboarding_pnp = \
                DeviceOnboardingPnp_v2_2_3_3(
                    self._session, object_factory, _validator
                )
            self.device_replacement = \
                DeviceReplacement_v2_2_3_3(
                    self._session, object_factory, _validator
                )
            self.devices = \
                Devices_v2_2_3_3(
                    self._session, object_factory, _validator
                )
            self.disaster_recovery = \
                DisasterRecovery_v2_2_3_3(
                    self._session, object_factory, _validator
                )
            self.discovery = \
                Discovery_v2_2_3_3(
                    self._session, object_factory, _validator
                )
            self.event_management = \
                EventManagement_v2_2_3_3(
                    self._session, object_factory, _validator
                )
            self.fabric_wireless = \
                FabricWireless_v2_2_3_3(
                    self._session, object_factory, _validator
                )
            self.file = \
                File_v2_2_3_3(
                    self._session, object_factory, _validator
                )
            self.health_and_performance = \
                HealthAndPerformance_v2_2_3_3(
                    self._session, object_factory, _validator
                )
            self.itsm = \
                Itsm_v2_2_3_3(
                    self._session, object_factory, _validator
                )
            self.issues = \
                Issues_v2_2_3_3(
                    self._session, object_factory, _validator
                )
            self.licenses = \
                Licenses_v2_2_3_3(
                    self._session, object_factory, _validator
                )
            self.network_settings = \
                NetworkSettings_v2_2_3_3(
                    self._session, object_factory, _validator
                )
            self.path_trace = \
                PathTrace_v2_2_3_3(
                    self._session, object_factory, _validator
                )
            self.platform_configuration = \
                PlatformConfiguration_v2_2_3_3(
                    self._session, object_factory, _validator
                )
            self.policy = \
                Policy_v2_2_3_3(
                    self._session, object_factory, _validator
                )
            self.reports = \
                Reports_v2_2_3_3(
                    self._session, object_factory, _validator
                )
            self.sda = \
                Sda_v2_2_3_3(
                    self._session, object_factory, _validator
                )
            self.security_advisories = \
                SecurityAdvisories_v2_2_3_3(
                    self._session, object_factory, _validator
                )
            self.sensors = \
                Sensors_v2_2_3_3(
                    self._session, object_factory, _validator
                )
            self.site_design = \
                SiteDesign_v2_2_3_3(
                    self._session, object_factory, _validator
                )
            self.sites = \
                Sites_v2_2_3_3(
                    self._session, object_factory, _validator
                )
            self.software_image_management_swim = \
                SoftwareImageManagementSwim_v2_2_3_3(
                    self._session, object_factory, _validator
                )
            self.tag = \
                Tag_v2_2_3_3(
                    self._session, object_factory, _validator
                )
            self.task = \
                Task_v2_2_3_3(
                    self._session, object_factory, _validator
                )
            self.topology = \
                Topology_v2_2_3_3(
                    self._session, object_factory, _validator
                )
            self.users = \
                Users_v2_2_3_3(
                    self._session, object_factory, _validator
                )
            self.wireless = \
                Wireless_v2_2_3_3(
                    self._session, object_factory, _validator
                )
        if version == '2.3.3.0':
            self.application_policy = \
                ApplicationPolicy_v2_3_3_0(
                    self._session, object_factory, _validator
                )
            self.applications = \
                Applications_v2_3_3_0(
                    self._session, object_factory, _validator
                )
            self.cisco_dna_center_system = \
                CiscoDnaCenterSystem_v2_3_3_0(
                    self._session, object_factory, _validator
                )
            self.clients = \
                Clients_v2_3_3_0(
                    self._session, object_factory, _validator
                )
            self.command_runner = \
                CommandRunner_v2_3_3_0(
                    self._session, object_factory, _validator
                )
            self.compliance = \
                Compliance_v2_3_3_0(
                    self._session, object_factory, _validator
                )
            self.configuration_archive = \
                ConfigurationArchive_v2_3_3_0(
                    self._session, object_factory, _validator
                )
            self.configuration_templates = \
                ConfigurationTemplates_v2_3_3_0(
                    self._session, object_factory, _validator
                )
            self.device_onboarding_pnp = \
                DeviceOnboardingPnp_v2_3_3_0(
                    self._session, object_factory, _validator
                )
            self.device_replacement = \
                DeviceReplacement_v2_3_3_0(
                    self._session, object_factory, _validator
                )
            self.devices = \
                Devices_v2_3_3_0(
                    self._session, object_factory, _validator
                )
            self.discovery = \
                Discovery_v2_3_3_0(
                    self._session, object_factory, _validator
                )
            self.event_management = \
                EventManagement_v2_3_3_0(
                    self._session, object_factory, _validator
                )
            self.fabric_wireless = \
                FabricWireless_v2_3_3_0(
                    self._session, object_factory, _validator
                )
            self.file = \
                File_v2_3_3_0(
                    self._session, object_factory, _validator
                )
            self.health_and_performance = \
                HealthAndPerformance_v2_3_3_0(
                    self._session, object_factory, _validator
                )
            self.itsm = \
                Itsm_v2_3_3_0(
                    self._session, object_factory, _validator
                )
            self.issues = \
                Issues_v2_3_3_0(
                    self._session, object_factory, _validator
                )
            self.lan_automation = \
                LanAutomation_v2_3_3_0(
                    self._session, object_factory, _validator
                )
            self.licenses = \
                Licenses_v2_3_3_0(
                    self._session, object_factory, _validator
                )
            self.network_settings = \
                NetworkSettings_v2_3_3_0(
                    self._session, object_factory, _validator
                )
            self.path_trace = \
                PathTrace_v2_3_3_0(
                    self._session, object_factory, _validator
                )
            self.platform_configuration = \
                PlatformConfiguration_v2_3_3_0(
                    self._session, object_factory, _validator
                )
            self.reports = \
                Reports_v2_3_3_0(
                    self._session, object_factory, _validator
                )
            self.sda = \
                Sda_v2_3_3_0(
                    self._session, object_factory, _validator
                )
            self.security_advisories = \
                SecurityAdvisories_v2_3_3_0(
                    self._session, object_factory, _validator
                )
            self.sensors = \
                Sensors_v2_3_3_0(
                    self._session, object_factory, _validator
                )
            self.site_design = \
                SiteDesign_v2_3_3_0(
                    self._session, object_factory, _validator
                )
            self.sites = \
                Sites_v2_3_3_0(
                    self._session, object_factory, _validator
                )
            self.software_image_management_swim = \
                SoftwareImageManagementSwim_v2_3_3_0(
                    self._session, object_factory, _validator
                )
            self.system_settings = \
                SystemSettings_v2_3_3_0(
                    self._session, object_factory, _validator
                )
            self.tag = \
                Tag_v2_3_3_0(
                    self._session, object_factory, _validator
                )
            self.task = \
                Task_v2_3_3_0(
                    self._session, object_factory, _validator
                )
            self.topology = \
                Topology_v2_3_3_0(
                    self._session, object_factory, _validator
                )
            self.users = \
                Users_v2_3_3_0(
                    self._session, object_factory, _validator
                )
            self.wireless = \
                Wireless_v2_3_3_0(
                    self._session, object_factory, _validator
                )
        if version == '2.3.5.3':
            self.application_policy = \
                ApplicationPolicy_v2_3_5_3(
                    self._session, object_factory, _validator
                )
            self.applications = \
                Applications_v2_3_5_3(
                    self._session, object_factory, _validator
                )
            self.authentication_management = \
                AuthenticationManagement_v2_3_5_3(
                    self._session, object_factory, _validator
                )
            self.cisco_dna_center_system = \
                CiscoDnaCenterSystem_v2_3_5_3(
                    self._session, object_factory, _validator
                )
            self.clients = \
                Clients_v2_3_5_3(
                    self._session, object_factory, _validator
                )
            self.command_runner = \
                CommandRunner_v2_3_5_3(
                    self._session, object_factory, _validator
                )
            self.compliance = \
                Compliance_v2_3_5_3(
                    self._session, object_factory, _validator
                )
            self.configuration_archive = \
                ConfigurationArchive_v2_3_5_3(
                    self._session, object_factory, _validator
                )
            self.configuration_templates = \
                ConfigurationTemplates_v2_3_5_3(
                    self._session, object_factory, _validator
                )
            self.device_onboarding_pnp = \
                DeviceOnboardingPnp_v2_3_5_3(
                    self._session, object_factory, _validator
                )
            self.device_replacement = \
                DeviceReplacement_v2_3_5_3(
                    self._session, object_factory, _validator
                )
            self.devices = \
                Devices_v2_3_5_3(
                    self._session, object_factory, _validator
                )
            self.discovery = \
                Discovery_v2_3_5_3(
                    self._session, object_factory, _validator
                )
            self.eox = \
                Eox_v2_3_5_3(
                    self._session, object_factory, _validator
                )
            self.event_management = \
                EventManagement_v2_3_5_3(
                    self._session, object_factory, _validator
                )
            self.fabric_wireless = \
                FabricWireless_v2_3_5_3(
                    self._session, object_factory, _validator
                )
            self.file = \
                File_v2_3_5_3(
                    self._session, object_factory, _validator
                )
            self.health_and_performance = \
                HealthAndPerformance_v2_3_5_3(
                    self._session, object_factory, _validator
                )
            self.itsm = \
                Itsm_v2_3_5_3(
                    self._session, object_factory, _validator
                )
            self.itsm_integration = \
                ItsmIntegration_v2_3_5_3(
                    self._session, object_factory, _validator
                )
            self.issues = \
                Issues_v2_3_5_3(
                    self._session, object_factory, _validator
                )
            self.lan_automation = \
                LanAutomation_v2_3_5_3(
                    self._session, object_factory, _validator
                )
            self.licenses = \
                Licenses_v2_3_5_3(
                    self._session, object_factory, _validator
                )
            self.network_settings = \
                NetworkSettings_v2_3_5_3(
                    self._session, object_factory, _validator
                )
            self.path_trace = \
                PathTrace_v2_3_5_3(
                    self._session, object_factory, _validator
                )
            self.platform = \
                Platform_v2_3_5_3(
                    self._session, object_factory, _validator
                )
            self.platform_configuration = \
                Platform_v2_3_5_3(
                    self._session, object_factory, _validator
                )
            self.reports = \
                Reports_v2_3_5_3(
                    self._session, object_factory, _validator
                )
            self.sda = \
                Sda_v2_3_5_3(
                    self._session, object_factory, _validator
                )
            self.security_advisories = \
                SecurityAdvisories_v2_3_5_3(
                    self._session, object_factory, _validator
                )
            self.sensors = \
                Sensors_v2_3_5_3(
                    self._session, object_factory, _validator
                )
            self.site_design = \
                SiteDesign_v2_3_5_3(
                    self._session, object_factory, _validator
                )
            self.sites = \
                Sites_v2_3_5_3(
                    self._session, object_factory, _validator
                )
            self.software_image_management_swim = \
                SoftwareImageManagementSwim_v2_3_5_3(
                    self._session, object_factory, _validator
                )
            self.system_settings = \
                SystemSettings_v2_3_5_3(
                    self._session, object_factory, _validator
                )
            self.tag = \
                Tag_v2_3_5_3(
                    self._session, object_factory, _validator
                )
            self.task = \
                Task_v2_3_5_3(
                    self._session, object_factory, _validator
                )
            self.topology = \
                Topology_v2_3_5_3(
                    self._session, object_factory, _validator
                )
            self.user_and_roles = \
                UserandRoles_v2_3_5_3(
                    self._session, object_factory, _validator
                )
            self.users = \
                Users_v2_3_5_3(
                    self._session, object_factory, _validator
                )
            self.wireless = \
                Wireless_v2_3_5_3(
                    self._session, object_factory, _validator
                )

        if version == '2.3.7.6':

            self.ai_endpoint_analytics = \
                AIEndpointAnalytics_v2_3_7_6(
                    self._session, object_factory, _validator
                )
            self.application_policy = \
                ApplicationPolicy_v2_3_7_6(
                    self._session, object_factory, _validator
                )
            self.applications = \
                Applications_v2_3_7_6(
                    self._session, object_factory, _validator
                )
            self.authentication_management = \
                AuthenticationManagement_v2_3_7_6(
                    self._session, object_factory, _validator
                )
            self.cisco_trusted_certificates = \
                CiscoTrustedCertificates_v2_3_7_6(
                    self._session, object_factory, _validator
                )
            self.clients = \
                Clients_v2_3_7_6(
                    self._session, object_factory, _validator
                )
            self.command_runner = \
                CommandRunner_v2_3_7_6(
                    self._session, object_factory, _validator
                )
            self.compliance = \
                Compliance_v2_3_7_6(
                    self._session, object_factory, _validator
                )
            self.configuration_archive = \
                ConfigurationArchive_v2_3_7_6(
                    self._session, object_factory, _validator
                )
            self.configuration_templates = \
                ConfigurationTemplates_v2_3_7_6(
                    self._session, object_factory, _validator
                )
            self.device_onboarding_pnp = \
                DeviceOnboardingPnp_v2_3_7_6(
                    self._session, object_factory, _validator
                )
            self.device_replacement = \
                DeviceReplacement_v2_3_7_6(
                    self._session, object_factory, _validator
                )
            self.devices = \
                Devices_v2_3_7_6(
                    self._session, object_factory, _validator
                )
            self.disaster_recovery = \
                DisasterRecovery_v2_3_7_6(
                    self._session, object_factory, _validator
                )
            self.discovery = \
                Discovery_v2_3_7_6(
                    self._session, object_factory, _validator
                )
            self.eox = \
                EoX_v2_3_7_6(
                    self._session, object_factory, _validator
                )
            self.event_management = \
                EventManagement_v2_3_7_6(
                    self._session, object_factory, _validator
                )
            self.fabric_wireless = \
                FabricWireless_v2_3_7_6(
                    self._session, object_factory, _validator
                )
            self.file = \
                File_v2_3_7_6(
                    self._session, object_factory, _validator
                )
            self.health_and_performance = \
                HealthAndPerformance_v2_3_7_6(
                    self._session, object_factory, _validator
                )
            self.itsm = \
                Itsm_v2_3_7_6(
                    self._session, object_factory, _validator
                )
            self.itsm_integration = \
                ItsmIntegration_v2_3_7_6(
                    self._session, object_factory, _validator
                )
            self.issues = \
                Issues_v2_3_7_6(
                    self._session, object_factory, _validator
                )
            self.lan_automation = \
                LanAutomation_v2_3_7_6(
                    self._session, object_factory, _validator
                )
            self.licenses = \
                Licenses_v2_3_7_6(
                    self._session, object_factory, _validator
                )
            self.network_settings = \
                NetworkSettings_v2_3_7_6(
                    self._session, object_factory, _validator
                )
            self.path_trace = \
                PathTrace_v2_3_7_6(
                    self._session, object_factory, _validator
                )
            self.platform = \
                Platform_v2_3_7_6(
                    self._session, object_factory, _validator
                )
            self.reports = \
                Reports_v2_3_7_6(
                    self._session, object_factory, _validator
                )
            self.sda = \
                Sda_v2_3_7_6(
                    self._session, object_factory, _validator
                )
            self.security_advisories = \
                SecurityAdvisories_v2_3_7_6(
                    self._session, object_factory, _validator
                )
            self.sensors = \
                Sensors_v2_3_7_6(
                    self._session, object_factory, _validator
                )
            self.site_design = \
                SiteDesign_v2_3_7_6(
                    self._session, object_factory, _validator
                )
            self.sites = \
                Sites_v2_3_7_6(
                    self._session, object_factory, _validator
                )
            self.software_image_management_swim = \
                SoftwareImageManagementSwim_v2_3_7_6(
                    self._session, object_factory, _validator
                )
            self.system_settings = \
                SystemSettings_v2_3_7_6(
                    self._session, object_factory, _validator
                )
            self.tag = \
                Tag_v2_3_7_6(
                    self._session, object_factory, _validator
                )
            self.task = \
                Task_v2_3_7_6(
                    self._session, object_factory, _validator
                )
            self.topology = \
                Topology_v2_3_7_6(
                    self._session, object_factory, _validator
                )
            self.user_and_roles = \
                UserandRoles_v2_3_7_6(
                    self._session, object_factory, _validator
                )
            self.users = \
                Users_v2_3_7_6(
                    self._session, object_factory, _validator
                )
            self.wireless = \
                Wireless_v2_3_7_6(
                    self._session, object_factory, _validator
                )

        if version == '2.3.7.9':
            self.a_i_endpoint_analytics = \
                AIEndpointAnalytics_v2_3_7_9(
                    self._session, object_factory, _validator
                )
            self.application_policy = \
                ApplicationPolicy_v2_3_7_9(
                    self._session, object_factory, _validator
                )
            self.applications = \
                Applications_v2_3_7_9(
                    self._session, object_factory, _validator
                )
            self.authentication_management = \
                AuthenticationManagement_v2_3_7_9(
                    self._session, object_factory, _validator
                )
            self.cisco_i_m_c = \
                CiscoIMC_v2_3_7_9(
                    self._session, object_factory, _validator
                )
            self.cisco_trusted_certificates = \
                CiscoTrustedCertificates_v2_3_7_9(
                    self._session, object_factory, _validator
                )
            self.clients = \
                Clients_v2_3_7_9(
                    self._session, object_factory, _validator
                )
            self.command_runner = \
                CommandRunner_v2_3_7_9(
                    self._session, object_factory, _validator
                )
            self.compliance = \
                Compliance_v2_3_7_9(
                    self._session, object_factory, _validator
                )
            self.configuration_archive = \
                ConfigurationArchive_v2_3_7_9(
                    self._session, object_factory, _validator
                )
            self.configuration_templates = \
                ConfigurationTemplates_v2_3_7_9(
                    self._session, object_factory, _validator
                )
            self.device_onboarding_pnp = \
                DeviceOnboardingPnp_v2_3_7_9(
                    self._session, object_factory, _validator
                )
            self.device_replacement = \
                DeviceReplacement_v2_3_7_9(
                    self._session, object_factory, _validator
                )
            self.devices = \
                Devices_v2_3_7_9(
                    self._session, object_factory, _validator
                )
            self.disaster_recovery = \
                DisasterRecovery_v2_3_7_9(
                    self._session, object_factory, _validator
                )
            self.discovery = \
                Discovery_v2_3_7_9(
                    self._session, object_factory, _validator
                )
            self.eox = \
                EoX_v2_3_7_9(
                    self._session, object_factory, _validator
                )
            self.event_management = \
                EventManagement_v2_3_7_9(
                    self._session, object_factory, _validator
                )
            self.fabric_wireless = \
                FabricWireless_v2_3_7_9(
                    self._session, object_factory, _validator
                )
            self.file = \
                File_v2_3_7_9(
                    self._session, object_factory, _validator
                )
            self.health_and_performance = \
                HealthAndPerformance_v2_3_7_9(
                    self._session, object_factory, _validator
                )
            self.itsm = \
                Itsm_v2_3_7_9(
                    self._session, object_factory, _validator
                )
            self.itsm_integration = \
                ItsmIntegration_v2_3_7_9(
                    self._session, object_factory, _validator
                )
            self.issues = \
                Issues_v2_3_7_9(
                    self._session, object_factory, _validator
                )
            self.lan_automation = \
                LanAutomation_v2_3_7_9(
                    self._session, object_factory, _validator
                )
            self.licenses = \
                Licenses_v2_3_7_9(
                    self._session, object_factory, _validator
                )
            self.network_settings = \
                NetworkSettings_v2_3_7_9(
                    self._session, object_factory, _validator
                )
            self.path_trace = \
                PathTrace_v2_3_7_9(
                    self._session, object_factory, _validator
                )
            self.platform = \
                Platform_v2_3_7_9(
                    self._session, object_factory, _validator
                )
            self.reports = \
                Reports_v2_3_7_9(
                    self._session, object_factory, _validator
                )
            self.sda = \
                Sda_v2_3_7_9(
                    self._session, object_factory, _validator
                )
            self.security_advisories = \
                SecurityAdvisories_v2_3_7_9(
                    self._session, object_factory, _validator
                )
            self.sensors = \
                Sensors_v2_3_7_9(
                    self._session, object_factory, _validator
                )
            self.site_design = \
                SiteDesign_v2_3_7_9(
                    self._session, object_factory, _validator
                )
            self.sites = \
                Sites_v2_3_7_9(
                    self._session, object_factory, _validator
                )
            self.software_image_management_swim = \
                SoftwareImageManagementSwim_v2_3_7_9(
                    self._session, object_factory, _validator
                )
            self.system_settings = \
                SystemSettings_v2_3_7_9(
                    self._session, object_factory, _validator
                )
            self.tag = \
                Tag_v2_3_7_9(
                    self._session, object_factory, _validator
                )
            self.task = \
                Task_v2_3_7_9(
                    self._session, object_factory, _validator
                )
            self.topology = \
                Topology_v2_3_7_9(
                    self._session, object_factory, _validator
                )
            self.userand_roles = \
                UserandRoles_v2_3_7_9(
                    self._session, object_factory, _validator
                )
            self.users = \
                Users_v2_3_7_9(
                    self._session, object_factory, _validator
                )
            self.wireless = \
                Wireless_v2_3_7_9(
                    self._session, object_factory, _validator
                )
        self.custom_caller = \
            CustomCaller(self._session, object_factory)

    @property
    def session(self):
        """The DNA Center API session."""
        return self._session

    @property
    def access_token(self):
        """The access token used for API calls to the DNA Center service."""
        return self._session.access_token

    @property
    def base_url(self):
        """The base URL prefixed to the individual API endpoint suffixes."""
        return self._session.base_url

    @property
    def single_request_timeout(self):
        """Timeout (in seconds) for an single HTTP request."""
        return self._session.single_request_timeout

    @property
    def wait_on_rate_limit(self):
        """Automatic rate-limit handling enabled / disabled."""
        return self._session.wait_on_rate_limit

    @property
    def verify(self):
        """The verify (TLS Certificate) for the API endpoints."""
        return self._session._verify

    @property
    def version(self):
        """The API version of DNA Center."""
        return self._session._version

    @verify.setter
    def verify(self, value):
        """The verify (TLS Certificate) for the API endpoints."""
        self.authentication.verify = value
        self._session.verify = value

    @base_url.setter
    def base_url(self, value):
        """The base URL for the API endpoints."""
        self._session.base_url = value

    @single_request_timeout.setter
    def single_request_timeout(self, value):
        """The timeout (seconds) for a single HTTP REST API request."""
        self.authentication.single_request_timeout = value
        self._session.single_request_timeout = value

    @wait_on_rate_limit.setter
    def wait_on_rate_limit(self, value):
        """Enable or disable automatic rate-limit handling."""
        self._session.wait_on_rate_limit = value



# -*- coding: utf-8 -*-
"""Package exceptions.

Copyright (c) 2019-2021 Cisco Systems.

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""


import logging
from builtins import *

import requests

from .response_codes import RESPONSE_CODES

logger = logging.getLogger(__name__)


class dnacentersdkException(Exception):
    """Base class for all dnacentersdk package exceptions."""
    pass


class AccessTokenError(dnacentersdkException):
    """Raised when an incorrect DNA Center Access Token has been provided."""
    pass


class VersionError(dnacentersdkException):
    """Raised when an incorrect DNA Center version has been provided."""
    pass


class DownloadFailure(dnacentersdkException):
    """Errors returned in response to requests sent to the DNA Center APIs
    with stream=True.

    Several data attributes are available for inspection.
    """
    def __init__(self, response, exception):
        assert isinstance(response, requests.Response)
        assert isinstance(exception, Exception)

        # Extended exception attributes
        self.response = response
        """The :class:`requests.Response` object returned from the API call."""

        self.request = self.response.request
        """The :class:`requests.PreparedRequest` of the API call."""

        self.status_code = self.response.status_code
        """The HTTP status code from the API response."""

        self.status = self.response.reason
        """The HTTP status from the API response."""

        self.original_error = exception
        """The original exception"""

        self.raw = self.response.raw
        """The raw value of the API response"""

        self.message = "Check raw property to retrieve raw response."

        super(DownloadFailure, self).__init__(
            "[{status_code}]{status} - {message} : {original_error}".format(
                status_code=self.status_code,
                status=" " + self.status if self.status else "",
                message=self.message,
                original_error=self.original_error,
            )
        )

    def __repr__(self):
        return "<{exception_name} [{status_code}]>".format(
            exception_name=self.__class__.__name__,
            status_code=self.status_code,
        )


class ApiError(dnacentersdkException):
    """Errors returned in response to requests sent to the DNA Center APIs.

    Several data attributes are available for inspection.
    """

    def __init__(self, response):
        assert isinstance(response, requests.Response)

        # Extended exception attributes
        self.response = response
        """The :class:`requests.Response` object returned from the API call."""

        self.request = self.response.request
        """The :class:`requests.PreparedRequest` of the API call."""

        self.status_code = self.response.status_code
        """The HTTP status code from the API response."""

        self.status = self.response.reason
        """The HTTP status from the API response."""

        self.details = None
        """The parsed JSON details from the API response."""
        if "application/json" in \
                self.response.headers.get("Content-Type", "").lower():
            try:
                self.details = self.response.json()
            except ValueError:
                logger.warning("Error parsing JSON response body")

        # Extract detailed error information
        error_code = self.details.get("response", {}).get("errorCode") if self.details else None
        message = self.details.get("response", {}).get("message") if self.details else None
        detail = self.details.get("response", {}).get("detail") if self.details else None

        # Construct the error message
        self.message = f"{error_code} - {message}: {detail}" if error_code and message and detail else \
            message or self.details.get("description") if self.details and isinstance(self.details, dict) else None
        """The error message from the parsed API response."""

        self.description = RESPONSE_CODES.get(self.status_code)
        """A description of the HTTP Response Code from the API docs."""

        super(ApiError, self).__init__(
            "[{status_code}]{status} - {message}".format(
                status_code=self.status_code,
                status=" " + self.status if self.status else "",
                message=self.message or self.description or "Unknown Error",
            )
        )

    def __repr__(self):
        return "<{exception_name} [{status_code}]>".format(
            exception_name=self.__class__.__name__,
            status_code=self.status_code,
        )


class RateLimitError(ApiError):
    """Cisco DNA Center Rate-Limit exceeded Error.

    Raised when a rate-limit exceeded message is received and the request
    **will not** be retried.
    """

    def __init__(self, response):
        assert isinstance(response, requests.Response)

        # Extended exception attributes
        self.retry_after = max(1, int(response.headers.get('Retry-After', 15)))
        """The `Retry-After` time period (in seconds) provided by DNA Center.

        Defaults to 15 seconds if the response `Retry-After` header isn't
        present in the response headers, and defaults to a minimum wait time of
        1 second if DNA Center returns a `Retry-After` header of 0 seconds.
        """

        super(RateLimitError, self).__init__(response)


class RateLimitWarning(UserWarning):
    """Cisco DNA Center rate-limit exceeded warning.

    Raised when a rate-limit exceeded message is received and the request will
    be retried.
    """

    def __init__(self, response):
        assert isinstance(response, requests.Response)

        # Extended warning attributes
        self.retry_after = max(1, int(response.headers.get('Retry-After', 15)))
        """The `Retry-After` time period (in seconds) provided by DNA Center.

        Defaults to 15 seconds if the response `Retry-After` header isn't
        present in the response headers, and defaults to a minimum wait time of
        1 second if DNA Center returns a `Retry-After` header of 0 seconds.
        """

        super(RateLimitWarning, self).__init__()


class MalformedRequest(dnacentersdkException):
    """Raised when a malformed request is received from DNA Center user."""
    pass



# -*- coding: utf-8 -*-
"""Cisco Catalyst Center Fabric Wireless API wrapper.

Copyright (c) 2024 Cisco Systems.

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""



from builtins import *



from ...restsession import RestSession
from ...utils import (
    apply_path_params,
    check_type,
    dict_from_items_with_values,
    dict_of_str,
)


class FabricWireless(object):
    """Cisco Catalyst Center Fabric Wireless API (version: 2.3.7.9).

    Wraps the Catalyst Center Fabric Wireless
    API and exposes the API as native Python
    methods that return native Python objects.

    """

    def __init__(self, session, object_factory, request_validator):
        """Initialize a new FabricWireless
        object with the provided RestSession.

        Args:
            session(RestSession): The RESTful session object to be used for
                API calls to the Catalyst Center service.

        Raises:
            TypeError: If the parameter types are incorrect.

        """
        check_type(session, RestSession)

        super(FabricWireless, self).__init__()

        self._session = session
        self._object_factory = object_factory
        self._request_validator = request_validator

    def add_ssid_to_ip_pool_mapping_v1(self,
                                       scalableGroupName=None,
                                       siteNameHierarchy=None,
                                       ssidNames=None,
                                       vlanName=None,
                                       headers=None,
                                       payload=None,
                                       active_validation=True,
                                       **request_parameters):
        """Add SSID to IP Pool Mapping .

        Args:
            scalableGroupName(string): Fabric Wireless's Scalable Group Name .
            siteNameHierarchy(string): Fabric Wireless's Site Name Hierarchy .
            ssidNames(list): Fabric Wireless's List of SSIDs  (list of strings).
            vlanName(string): Fabric Wireless's VLAN Name .
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            payload(dict): A JSON serializable Python object to send in the
                body of the Request.
            active_validation(bool): Enable/Disable payload validation.
                Defaults to True.
            **request_parameters: Additional request parameters (provides
                support for parameters that may be added in the future).

        Returns:
            MyDict: JSON response. Access the object's properties by using
            the dot notation or the bracket notation.

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the Catalyst Center cloud returns an error.
        Documentation Link:
            https://developer.cisco.com/docs/dna-center/#!add-s-s-i-d-to-i-p-pool-mapping
        """
        check_type(headers, dict)
        check_type(payload, dict)
        if headers is not None:
            if 'X-Auth-Token' in headers:
                check_type(headers.get('X-Auth-Token'),
                           str, may_be_none=False)

        _params = {
        }
        _params.update(request_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
        }
        _payload = {
            'vlanName':
                vlanName,
            'scalableGroupName':
                scalableGroupName,
            'ssidNames':
                ssidNames,
            'siteNameHierarchy':
                siteNameHierarchy,
        }
        _payload.update(payload or {})
        _payload = dict_from_items_with_values(_payload)
        if active_validation:
            self._request_validator('jsd_ad96e712f4525a128368b1bfe3afc21c_v2_3_7_9')\
                .validate(_payload)

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True

        e_url = ('/dna/intent/api/v1/business/sda/hostonboarding/ssid-'
                 + 'ippool')
        endpoint_full_url = apply_path_params(e_url, path_params)
        if with_custom_headers:
            json_data = self._session.post(endpoint_full_url, params=_params,
                                           json=_payload,
                                           headers=_headers)
        else:
            json_data = self._session.post(endpoint_full_url, params=_params,
                                           json=_payload)

        return self._object_factory('bpm_ad96e712f4525a128368b1bfe3afc21c_v2_3_7_9', json_data)

    def update_ssid_to_ip_pool_mapping_v1(self,
                                          scalableGroupName=None,
                                          siteNameHierarchy=None,
                                          ssidNames=None,
                                          vlanName=None,
                                          headers=None,
                                          payload=None,
                                          active_validation=True,
                                          **request_parameters):
        """Update SSID to IP Pool Mapping .

        Args:
            scalableGroupName(string): Fabric Wireless's Scalable Group Name .
            siteNameHierarchy(string): Fabric Wireless's Site Name Hierarchy .
            ssidNames(list): Fabric Wireless's List of SSIDs  (list of strings).
            vlanName(string): Fabric Wireless's VLAN Name .
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            payload(dict): A JSON serializable Python object to send in the
                body of the Request.
            active_validation(bool): Enable/Disable payload validation.
                Defaults to True.
            **request_parameters: Additional request parameters (provides
                support for parameters that may be added in the future).

        Returns:
            MyDict: JSON response. Access the object's properties by using
            the dot notation or the bracket notation.

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the Catalyst Center cloud returns an error.
        Documentation Link:
            https://developer.cisco.com/docs/dna-center/#!update-s-s-i-d-to-i-p-pool-mapping
        """
        check_type(headers, dict)
        check_type(payload, dict)
        if headers is not None:
            if 'X-Auth-Token' in headers:
                check_type(headers.get('X-Auth-Token'),
                           str, may_be_none=False)

        _params = {
        }
        _params.update(request_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
        }
        _payload = {
            'vlanName':
                vlanName,
            'scalableGroupName':
                scalableGroupName,
            'ssidNames':
                ssidNames,
            'siteNameHierarchy':
                siteNameHierarchy,
        }
        _payload.update(payload or {})
        _payload = dict_from_items_with_values(_payload)
        if active_validation:
            self._request_validator('jsd_f90ae8599c8a21c98b7a1ca804_v2_3_7_9')\
                .validate(_payload)

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True

        e_url = ('/dna/intent/api/v1/business/sda/hostonboarding/ssid-'
                 + 'ippool')
        endpoint_full_url = apply_path_params(e_url, path_params)
        if with_custom_headers:
            json_data = self._session.put(endpoint_full_url, params=_params,
                                          json=_payload,
                                          headers=_headers)
        else:
            json_data = self._session.put(endpoint_full_url, params=_params,
                                          json=_payload)

        return self._object_factory('bpm_f90ae8599c8a21c98b7a1ca804_v2_3_7_9', json_data)

    def get_ssid_to_ip_pool_mapping_v1(self,
                                       site_name_hierarchy,
                                       vlan_name,
                                       headers=None,
                                       **request_parameters):
        """Get SSID to IP Pool Mapping .

        Args:
            vlan_name(str): vlanName query parameter. VLAN Name .
            site_name_hierarchy(str): siteNameHierarchy query parameter. Site Name Heirarchy .
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            **request_parameters: Additional request parameters (provides
                support for parameters that may be added in the future).

        Returns:
            MyDict: JSON response. Access the object's properties by using
            the dot notation or the bracket notation.

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the Catalyst Center cloud returns an error.
        Documentation Link:
            https://developer.cisco.com/docs/dna-center/#!get-s-s-i-d-to-i-p-pool-mapping
        """
        check_type(headers, dict)
        check_type(vlan_name, str,
                   may_be_none=False)
        check_type(site_name_hierarchy, str,
                   may_be_none=False)
        if headers is not None:
            if 'X-Auth-Token' in headers:
                check_type(headers.get('X-Auth-Token'),
                           str, may_be_none=False)

        _params = {
            'vlanName':
                vlan_name,
            'siteNameHierarchy':
                site_name_hierarchy,
        }
        _params.update(request_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
        }

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True

        e_url = ('/dna/intent/api/v1/business/sda/hostonboarding/ssid-'
                 + 'ippool')
        endpoint_full_url = apply_path_params(e_url, path_params)
        if with_custom_headers:
            json_data = self._session.get(endpoint_full_url, params=_params,
                                          headers=_headers)
        else:
            json_data = self._session.get(endpoint_full_url, params=_params)

        return self._object_factory('bpm_b0f6a0410705c75a61cdc51cc96c53f_v2_3_7_9', json_data)

    def remove_w_l_c_from_fabric_domain_v1(self,
                                           device_ipaddress,
                                           headers=None,
                                           **request_parameters):
        """Remove WLC from Fabric Domain .

        Args:
            device_ipaddress(str): deviceIPAddress query parameter. Device Management IP Address .
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            **request_parameters: Additional request parameters (provides
                support for parameters that may be added in the future).

        Returns:
            MyDict: JSON response. Access the object's properties by using
            the dot notation or the bracket notation.

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the Catalyst Center cloud returns an error.
        Documentation Link:
            https://developer.cisco.com/docs/dna-center/#!remove-w-l-c-from-fabric-domain
        """
        check_type(headers, dict)
        check_type(device_ipaddress, str,
                   may_be_none=False)
        if headers is not None:
            if '__persistbapioutput' in headers:
                check_type(headers.get('__persistbapioutput'),
                           str)
            if 'X-Auth-Token' in headers:
                check_type(headers.get('X-Auth-Token'),
                           str, may_be_none=False)

        _params = {
            'deviceIPAddress':
                device_ipaddress,
        }
        _params.update(request_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
        }

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True

        e_url = ('/dna/intent/api/v1/business/sda/wireless-controller')
        endpoint_full_url = apply_path_params(e_url, path_params)
        if with_custom_headers:
            json_data = self._session.delete(endpoint_full_url, params=_params,
                                             headers=_headers)
        else:
            json_data = self._session.delete(endpoint_full_url, params=_params)

        return self._object_factory('bpm_bb706025a9cb183ce7a60e0b5df_v2_3_7_9', json_data)

    def add_w_l_c_to_fabric_domain_v1(self,
                                      deviceName=None,
                                      siteNameHierarchy=None,
                                      headers=None,
                                      payload=None,
                                      active_validation=True,
                                      **request_parameters):
        """Add WLC to Fabric Domain .

        Args:
            deviceName(string): Fabric Wireless's WLC Device Name .
            siteNameHierarchy(string): Fabric Wireless's Fabric Site Name Hierarchy .
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            payload(dict): A JSON serializable Python object to send in the
                body of the Request.
            active_validation(bool): Enable/Disable payload validation.
                Defaults to True.
            **request_parameters: Additional request parameters (provides
                support for parameters that may be added in the future).

        Returns:
            MyDict: JSON response. Access the object's properties by using
            the dot notation or the bracket notation.

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the Catalyst Center cloud returns an error.
        Documentation Link:
            https://developer.cisco.com/docs/dna-center/#!add-w-l-c-to-fabric-domain
        """
        check_type(headers, dict)
        check_type(payload, dict)
        if headers is not None:
            if 'X-Auth-Token' in headers:
                check_type(headers.get('X-Auth-Token'),
                           str, may_be_none=False)

        _params = {
        }
        _params.update(request_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
        }
        _payload = {
            'deviceName':
                deviceName,
            'siteNameHierarchy':
                siteNameHierarchy,
        }
        _payload.update(payload or {})
        _payload = dict_from_items_with_values(_payload)
        if active_validation:
            self._request_validator('jsd_c4befbd77a452a9b7873ffc360a1f20_v2_3_7_9')\
                .validate(_payload)

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True

        e_url = ('/dna/intent/api/v1/business/sda/wireless-controller')
        endpoint_full_url = apply_path_params(e_url, path_params)
        if with_custom_headers:
            json_data = self._session.post(endpoint_full_url, params=_params,
                                           json=_payload,
                                           headers=_headers)
        else:
            json_data = self._session.post(endpoint_full_url, params=_params,
                                           json=_payload)

        return self._object_factory('bpm_c4befbd77a452a9b7873ffc360a1f20_v2_3_7_9', json_data)

    def returns_all_the_fabric_sites_that_have_vlan_to_ssid_mapping_v1(self,
                                                                       limit=None,
                                                                       offset=None,
                                                                       headers=None,
                                                                       **request_parameters):
        """It will return all vlan to SSID mapping across all the fabric site .

        Args:
            limit(int): limit query parameter. Return only this many IP Pool to SSID Mapping. Default is 500 if not
                specified. Maximum allowed limit is 500. .
            offset(int): offset query parameter. Number of records to skip for pagination .
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            **request_parameters: Additional request parameters (provides
                support for parameters that may be added in the future).

        Returns:
            MyDict: JSON response. Access the object's properties by using
            the dot notation or the bracket notation.

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the Catalyst Center cloud returns an error.
        Documentation Link:
            https://developer.cisco.com/docs/dna-center/#!returns-all-the-fabric-sites-that-have-v-l-a-n-to-s-s-i-d-mapping
        """
        check_type(headers, dict)
        check_type(limit, int)
        check_type(offset, int)
        if headers is not None:
            if 'X-Auth-Token' in headers:
                check_type(headers.get('X-Auth-Token'),
                           str, may_be_none=False)

        _params = {
            'limit':
                limit,
            'offset':
                offset,
        }
        _params.update(request_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
        }

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True

        e_url = ('/dna/intent/api/v1/sda/fabrics/vlanToSsids')
        endpoint_full_url = apply_path_params(e_url, path_params)
        if with_custom_headers:
            json_data = self._session.get(endpoint_full_url, params=_params,
                                          headers=_headers)
        else:
            json_data = self._session.get(endpoint_full_url, params=_params)

        return self._object_factory('bpm_fea6e17769f5b3eb5ee1696254d2973_v2_3_7_9', json_data)

    def return_the_count_of_all_the_fabric_site_which_has_ssid_to_ip_pool_mapping_v1(self,
                                                                                     headers=None,
                                                                                     **request_parameters):
        """Return the count of all the fabric site which has SSID to IP Pool mapping .

        Args:
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            **request_parameters: Additional request parameters (provides
                support for parameters that may be added in the future).

        Returns:
            MyDict: JSON response. Access the object's properties by using
            the dot notation or the bracket notation.

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the Catalyst Center cloud returns an error.
        Documentation Link:
            https://developer.cisco.com/docs/dna-center/#!return-the-count-of-all-the-fabric-site-which-has-s-s-i-d-to-i-p-pool-mapping
        """
        check_type(headers, dict)
        if headers is not None:
            if 'X-Auth-Token' in headers:
                check_type(headers.get('X-Auth-Token'),
                           str, may_be_none=False)

        _params = {
        }
        _params.update(request_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
        }

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True

        e_url = ('/dna/intent/api/v1/sda/fabrics/vlanToSsids/count')
        endpoint_full_url = apply_path_params(e_url, path_params)
        if with_custom_headers:
            json_data = self._session.get(endpoint_full_url, params=_params,
                                          headers=_headers)
        else:
            json_data = self._session.get(endpoint_full_url, params=_params)

        return self._object_factory('bpm_be3f285e21b59701a1af044b28_v2_3_7_9', json_data)

    def switch_wireless_setting_and_rolling_ap_upgrade_management_v1(self,
                                                                     fabric_id,
                                                                     enableWireless=None,
                                                                     id=None,
                                                                     rollingApUpgrade=None,
                                                                     headers=None,
                                                                     payload=None,
                                                                     active_validation=True,
                                                                     **request_parameters):
        """This API is used to enable or disable wireless capabilities on switch devices, along with configuring rolling AP
        upgrades on the fabric site. Reboot action is required to remove wireless configurations. .

        Args:
            enableWireless(boolean): Fabric Wireless's Enable Wireless.
            id(string): Fabric Wireless's Network Device ID of the wireless capable switch .
            rollingApUpgrade(object): Fabric Wireless's rollingApUpgrade.
            fabric_id(str): fabricId path parameter. The 'fabricId' represents the Fabric ID of a particular
                Fabric Site. The 'fabricId' can be obtained using the api
                /dna/intent/api/v1/sda/fabricSites.  Example : e290f1ee-6c54-4b01-90e6-d701748f0851 .
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            payload(dict): A JSON serializable Python object to send in the
                body of the Request.
            active_validation(bool): Enable/Disable payload validation.
                Defaults to True.
            **request_parameters: Additional request parameters (provides
                support for parameters that may be added in the future).

        Returns:
            MyDict: JSON response. Access the object's properties by using
            the dot notation or the bracket notation.

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the Catalyst Center cloud returns an error.
        Documentation Link:
            https://developer.cisco.com/docs/dna-center/#!switch-wireless-setting-and-rolling-a-p-upgrade-management
        """
        check_type(headers, dict)
        check_type(payload, dict)
        check_type(fabric_id, str,
                   may_be_none=False)
        if headers is not None:
            if 'X-Auth-Token' in headers:
                check_type(headers.get('X-Auth-Token'),
                           str, may_be_none=False)

        _params = {
        }
        _params.update(request_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
            'fabricId': fabric_id,
        }
        _payload = {
            'id':
                id,
            'enableWireless':
                enableWireless,
            'rollingApUpgrade':
                rollingApUpgrade,
        }
        _payload.update(payload or {})
        _payload = dict_from_items_with_values(_payload)
        if active_validation:
            self._request_validator('jsd_e5e51bcda0b5fec984ba8120f743fe2_v2_3_7_9')\
                .validate(_payload)

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True

        e_url = ('/dna/intent/api/v1/sda/fabrics/{fabricId}/switchWireless'
                 + 'Setting')
        endpoint_full_url = apply_path_params(e_url, path_params)
        if with_custom_headers:
            json_data = self._session.put(endpoint_full_url, params=_params,
                                          json=_payload,
                                          headers=_headers)
        else:
            json_data = self._session.put(endpoint_full_url, params=_params,
                                          json=_payload)

        return self._object_factory('bpm_e5e51bcda0b5fec984ba8120f743fe2_v2_3_7_9', json_data)

    def get_sda_wireless_details_from_switches_v1(self,
                                                  fabric_id,
                                                  headers=None,
                                                  **request_parameters):
        """Get the SDA Wireless details from the switches on the fabric site that have wireless capability enabled. A
        maximum of two switches can have a wireless role in a fabric site. .

        Args:
            fabric_id(str): fabricId path parameter. The 'fabricId' represents the Fabric ID of a particular
                Fabric Site. The 'fabricId' can be obtained using the api
                /dna/intent/api/v1/sda/fabricSites.  Example : e290f1ee-6c54-4b01-90e6-d701748f0851 .
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            **request_parameters: Additional request parameters (provides
                support for parameters that may be added in the future).

        Returns:
            MyDict: JSON response. Access the object's properties by using
            the dot notation or the bracket notation.

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the Catalyst Center cloud returns an error.
        Documentation Link:
            https://developer.cisco.com/docs/dna-center/#!get-s-d-a-wireless-details-from-switches
        """
        check_type(headers, dict)
        check_type(fabric_id, str,
                   may_be_none=False)
        if headers is not None:
            if 'X-Auth-Token' in headers:
                check_type(headers.get('X-Auth-Token'),
                           str, may_be_none=False)

        _params = {
        }
        _params.update(request_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
            'fabricId': fabric_id,
        }

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True

        e_url = ('/dna/intent/api/v1/sda/fabrics/{fabricId}/switchWireless'
                 + 'Setting')
        endpoint_full_url = apply_path_params(e_url, path_params)
        if with_custom_headers:
            json_data = self._session.get(endpoint_full_url, params=_params,
                                          headers=_headers)
        else:
            json_data = self._session.get(endpoint_full_url, params=_params)

        return self._object_factory('bpm_e33e204167d5408a6785177727f40c9_v2_3_7_9', json_data)

    def reload_switch_for_wireless_controller_cleanup_v1(self,
                                                         fabric_id,
                                                         deviceId=None,
                                                         headers=None,
                                                         payload=None,
                                                         active_validation=True,
                                                         **request_parameters):
        """This API is used to reload switches after disabling wireless to remove the wireless-controller configuration on
        the device. When wireless is disabled on a switch, all wireless configurations are removed except for
        the wireless-controller configuration. To completely remove the wireless-controller configuration, you
        can use this API. Please note that using this API will cause a reload of the device(s). This API should
        only be used for devices that have wireless disabled but still have the 'wireless-controller'
        configuration present. The reload payload can have a maximum of two switches as only two switches can
        have a wireless role in a fabric site. .

        Args:
            deviceId(string): Fabric Wireless's Network Device ID .
            fabric_id(str): fabricId path parameter. The 'fabricId' represents the Fabric ID of a particular
                Fabric Site. The 'fabricId' can be obtained using the api
                /dna/intent/api/v1/sda/fabricSites.  Example : e290f1ee-6c54-4b01-90e6-d701748f0851 .
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            payload(dict): A JSON serializable Python object to send in the
                body of the Request.
            active_validation(bool): Enable/Disable payload validation.
                Defaults to True.
            **request_parameters: Additional request parameters (provides
                support for parameters that may be added in the future).

        Returns:
            MyDict: JSON response. Access the object's properties by using
            the dot notation or the bracket notation.

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the Catalyst Center cloud returns an error.
        Documentation Link:
            https://developer.cisco.com/docs/dna-center/#!reload-switch-for-wireless-controller-cleanup
        """
        check_type(headers, dict)
        check_type(payload, dict)
        check_type(fabric_id, str,
                   may_be_none=False)
        if headers is not None:
            if 'X-Auth-Token' in headers:
                check_type(headers.get('X-Auth-Token'),
                           str, may_be_none=False)

        _params = {
        }
        _params.update(request_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
            'fabricId': fabric_id,
        }
        _payload = {
            'deviceId':
                deviceId,
        }
        _payload.update(payload or {})
        _payload = dict_from_items_with_values(_payload)
        if active_validation:
            self._request_validator('jsd_cdf8c0d3866d5147901c0cf4821a84a7_v2_3_7_9')\
                .validate(_payload)

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True

        e_url = ('/dna/intent/api/v1/sda/fabrics/{fabricId}/switchWireless'
                 + 'Setting/reload')
        endpoint_full_url = apply_path_params(e_url, path_params)
        if with_custom_headers:
            json_data = self._session.post(endpoint_full_url, params=_params,
                                           json=_payload,
                                           headers=_headers)
        else:
            json_data = self._session.post(endpoint_full_url, params=_params,
                                           json=_payload)

        return self._object_factory('bpm_cdf8c0d3866d5147901c0cf4821a84a7_v2_3_7_9', json_data)

    def add_update_or_remove_ssid_mapping_to_a_vlan_v1(self,
                                                       fabric_id,
                                                       headers=None,
                                                       payload=None,
                                                       active_validation=True,
                                                       **request_parameters):
        """Add, update, or remove SSID mappings to a VLAN. If the payload doesn't contain a 'vlanName' which has SSIDs
        mapping done earlier then all the mapped SSIDs of the 'vlanName' is cleared. The request must include
        all SSIDs currently mapped to a VLAN, as determined by the response from the GET operation for the same
        fabricId used in the request. If an already-mapped SSID is not included in the payload, its mapping will
        be removed by this API. Conversely, if a new SSID is provided, it will be added to the Mapping. Ensure
        that any new SSID added is a Fabric SSID. This API can also be used to add a VLAN and associate the
        relevant SSIDs with it. The 'vlanName' must be 'Fabric Wireless Enabled' and should be part of the
        Fabric Site representing 'Fabric ID' specified in the API request. .

        Args:
            fabric_id(str): fabricId path parameter. The 'fabricId' represents the Fabric ID of a particular
                Fabric Site .
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            payload(list): A JSON serializable Python object to send in the
                body of the Request.
            active_validation(bool): Enable/Disable payload validation.
                Defaults to True.
            **request_parameters: Additional request parameters (provides
                support for parameters that may be added in the future).

        Returns:
            MyDict: JSON response. Access the object's properties by using
            the dot notation or the bracket notation.

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the Catalyst Center cloud returns an error.
        Documentation Link:
            https://developer.cisco.com/docs/dna-center/#!add-update-or-remove-s-s-i-d-mapping-to-a-v-l-a-n
        """
        check_type(headers, dict)
        check_type(payload, list)
        check_type(fabric_id, str,
                   may_be_none=False)
        if headers is not None:
            if 'Content-Type' in headers:
                check_type(headers.get('Content-Type'),
                           str, may_be_none=False)
            if 'X-Auth-Token' in headers:
                check_type(headers.get('X-Auth-Token'),
                           str, may_be_none=False)

        _params = {
        }
        _params.update(request_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
            'fabricId': fabric_id,
        }
        _payload = payload or []
        if active_validation:
            self._request_validator('jsd_a3d2432ae8c55fe793c5180d8d5fce25_v2_3_7_9')\
                .validate(_payload)

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True

        e_url = ('/dna/intent/api/v1/sda/fabrics/{fabricId}/vlanToSsids')
        endpoint_full_url = apply_path_params(e_url, path_params)
        if with_custom_headers:
            json_data = self._session.put(endpoint_full_url, params=_params,
                                          json=_payload,
                                          headers=_headers)
        else:
            json_data = self._session.put(endpoint_full_url, params=_params,
                                          json=_payload)

        return self._object_factory('bpm_a3d2432ae8c55fe793c5180d8d5fce25_v2_3_7_9', json_data)

    def retrieve_the_vlans_and_ssids_mapped_to_the_vlan_within_a_fabric_site_v1(self,
                                                                                fabric_id,
                                                                                limit=None,
                                                                                offset=None,
                                                                                headers=None,
                                                                                **request_parameters):
        """Retrieve the VLANs and SSIDs mapped to the VLAN, within a Fabric Site. The 'fabricId' represents the Fabric ID
        of a particular Fabric Site. .

        Args:
            fabric_id(str): fabricId path parameter. The 'fabricId' represents the Fabric ID of a particular
                Fabric Site .
            limit(int): limit query parameter. The number of records to show for this page. Default is 500 if not
                specified. Maximum allowed limit is 500. .
            offset(int): offset query parameter. The first record to show for this page; the first record is
                numbered 1. .
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            **request_parameters: Additional request parameters (provides
                support for parameters that may be added in the future).

        Returns:
            MyDict: JSON response. Access the object's properties by using
            the dot notation or the bracket notation.

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the Catalyst Center cloud returns an error.
        Documentation Link:
            https://developer.cisco.com/docs/dna-center/#!retrieve-the-v-l-a-ns-and-s-s-i-ds-mapped-to-the-v-l-a-n-within-a-fabric-site
        """
        check_type(headers, dict)
        check_type(limit, int)
        check_type(offset, int)
        check_type(fabric_id, str,
                   may_be_none=False)
        if headers is not None:
            if 'X-Auth-Token' in headers:
                check_type(headers.get('X-Auth-Token'),
                           str, may_be_none=False)

        _params = {
            'limit':
                limit,
            'offset':
                offset,
        }
        _params.update(request_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
            'fabricId': fabric_id,
        }

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True

        e_url = ('/dna/intent/api/v1/sda/fabrics/{fabricId}/vlanToSsids')
        endpoint_full_url = apply_path_params(e_url, path_params)
        if with_custom_headers:
            json_data = self._session.get(endpoint_full_url, params=_params,
                                          headers=_headers)
        else:
            json_data = self._session.get(endpoint_full_url, params=_params)

        return self._object_factory('bpm_a18f012c54a5d34aef05d651f2dea18_v2_3_7_9', json_data)

    def returns_the_count_of_vlans_mapped_to_ssids_in_a_fabric_site_v1(self,
                                                                       fabric_id,
                                                                       headers=None,
                                                                       **request_parameters):
        """Returns the count of VLANs mapped to SSIDs in a Fabric Site. The 'fabricId' represents the Fabric ID of a
        particular Fabric Site. .

        Args:
            fabric_id(str): fabricId path parameter. The 'fabricId' represents the Fabric ID of a particular
                Fabric Site .
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            **request_parameters: Additional request parameters (provides
                support for parameters that may be added in the future).

        Returns:
            MyDict: JSON response. Access the object's properties by using
            the dot notation or the bracket notation.

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the Catalyst Center cloud returns an error.
        Documentation Link:
            https://developer.cisco.com/docs/dna-center/#!returns-the-count-of-v-l-a-ns-mapped-to-s-s-i-ds-in-a-fabric-site
        """
        check_type(headers, dict)
        check_type(fabric_id, str,
                   may_be_none=False)
        if headers is not None:
            if 'Content-Type' in headers:
                check_type(headers.get('Content-Type'),
                           str, may_be_none=False)
            if 'X-Auth-Token' in headers:
                check_type(headers.get('X-Auth-Token'),
                           str, may_be_none=False)

        _params = {
        }
        _params.update(request_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
            'fabricId': fabric_id,
        }

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True

        e_url = ('/dna/intent/api/v1/sda/fabrics/{fabricId}/vlanToSsids/co'
                 + 'unt')
        endpoint_full_url = apply_path_params(e_url, path_params)
        if with_custom_headers:
            json_data = self._session.get(endpoint_full_url, params=_params,
                                          headers=_headers)
        else:
            json_data = self._session.get(endpoint_full_url, params=_params)

        return self._object_factory('bpm_ed14be6211da53ab832acf9b5aea599c_v2_3_7_9', json_data)

    def update_sda_wireless_multicast_v1(self,
                                         fabric_id,
                                         multicastEnabled=None,
                                         headers=None,
                                         payload=None,
                                         active_validation=True,
                                         **request_parameters):
        """Updates the Software-Defined Access (SDA) Wireless Multicast setting for a specified fabric site. This API
        allows you to enable or disable the multicast feature. For optimal performance, ensure wired multicast
        is also enabled. .

        Args:
            multicastEnabled(boolean): Fabric Wireless's Multicast Enabled.
            fabric_id(str): fabricId path parameter. The unique identifier of the fabric site for which the
                multicast setting is being requested. The identifier should be in the format of a UUID.
                The 'fabricId' can be obtained using the api /dna/intent/api/v1/sda/fabricSites. .
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            payload(dict): A JSON serializable Python object to send in the
                body of the Request.
            active_validation(bool): Enable/Disable payload validation.
                Defaults to True.
            **request_parameters: Additional request parameters (provides
                support for parameters that may be added in the future).

        Returns:
            MyDict: JSON response. Access the object's properties by using
            the dot notation or the bracket notation.

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the Catalyst Center cloud returns an error.
        Documentation Link:
            https://developer.cisco.com/docs/dna-center/#!update-s-d-a-wireless-multicast
        """
        check_type(headers, dict)
        check_type(payload, dict)
        check_type(fabric_id, str,
                   may_be_none=False)
        if headers is not None:
            if 'X-Auth-Token' in headers:
                check_type(headers.get('X-Auth-Token'),
                           str, may_be_none=False)

        _params = {
        }
        _params.update(request_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
            'fabricId': fabric_id,
        }
        _payload = {
            'multicastEnabled':
                multicastEnabled,
        }
        _payload.update(payload or {})
        _payload = dict_from_items_with_values(_payload)
        if active_validation:
            self._request_validator('jsd_a2be6dde4c587389e79d6cb84e54a6_v2_3_7_9')\
                .validate(_payload)

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True

        e_url = ('/dna/intent/api/v1/sda/fabrics/{fabricId}/wirelessMultic'
                 + 'ast')
        endpoint_full_url = apply_path_params(e_url, path_params)
        if with_custom_headers:
            json_data = self._session.put(endpoint_full_url, params=_params,
                                          json=_payload,
                                          headers=_headers)
        else:
            json_data = self._session.put(endpoint_full_url, params=_params,
                                          json=_payload)

        return self._object_factory('bpm_a2be6dde4c587389e79d6cb84e54a6_v2_3_7_9', json_data)

    def get_sda_wireless_multicast_v1(self,
                                      fabric_id,
                                      headers=None,
                                      **request_parameters):
        """Retrieves the current Software-Defined Access (SDA) Wireless Multicast setting for a specified fabric site. The
        setting indicates whether multicast is enabled (true) or disabled (false). For optimal performance,
        ensure wired multicast is also enabled. .

        Args:
            fabric_id(str): fabricId path parameter. The unique identifier of the fabric site for which the
                multicast setting is being requested. The identifier should be in the format of a UUID.
                The 'fabricId' can be obtained using the api /dna/intent/api/v1/sda/fabricSites. .
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            **request_parameters: Additional request parameters (provides
                support for parameters that may be added in the future).

        Returns:
            MyDict: JSON response. Access the object's properties by using
            the dot notation or the bracket notation.

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the Catalyst Center cloud returns an error.
        Documentation Link:
            https://developer.cisco.com/docs/dna-center/#!get-s-d-a-wireless-multicast
        """
        check_type(headers, dict)
        check_type(fabric_id, str,
                   may_be_none=False)
        if headers is not None:
            if 'X-Auth-Token' in headers:
                check_type(headers.get('X-Auth-Token'),
                           str, may_be_none=False)

        _params = {
        }
        _params.update(request_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
            'fabricId': fabric_id,
        }

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True

        e_url = ('/dna/intent/api/v1/sda/fabrics/{fabricId}/wirelessMultic'
                 + 'ast')
        endpoint_full_url = apply_path_params(e_url, path_params)
        if with_custom_headers:
            json_data = self._session.get(endpoint_full_url, params=_params,
                                          headers=_headers)
        else:
            json_data = self._session.get(endpoint_full_url, params=_params)

        return self._object_factory('bpm_fa08fad71522eb877d2356b584f7d_v2_3_7_9', json_data)



    # Alias Function
    def remove_w_l_c_from_fabric_domain(self,
                                           device_ipaddress,
                                           headers=None,
                                           **request_parameters):
        """ This function is an alias of remove_w_l_c_from_fabric_domain_v1 .
        Args:
            device_ipaddress(str): deviceIPAddress query parameter. Device Management IP Address .
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            **request_parameters: Additional request parameters (provides
                support for parameters that may be added in the future).

        Returns:
            This function returns the output of remove_w_l_c_from_fabric_domain_v1 .
        """
        return self.remove_w_l_c_from_fabric_domain_v1(
                    device_ipaddress=device_ipaddress,
                    headers=headers,
                    **request_parameters
        )


    # Alias Function
    def returns_all_the_fabric_sites_that_have_vlan_to_ssid_mapping(self,
                                                                       limit=None,
                                                                       offset=None,
                                                                       headers=None,
                                                                       **request_parameters):
        """ This function is an alias of returns_all_the_fabric_sites_that_have_vlan_to_ssid_mapping_v1 .
        Args:
            limit(int): limit query parameter. Return only this many IP Pool to SSID Mapping. Default is 500 if not
                specified. Maximum allowed limit is 500. .
            offset(int): offset query parameter. Number of records to skip for pagination .
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            **request_parameters: Additional request parameters (provides
                support for parameters that may be added in the future).

        Returns:
            This function returns the output of returns_all_the_fabric_sites_that_have_vlan_to_ssid_mapping_v1 .
        """
        return self.returns_all_the_fabric_sites_that_have_vlan_to_ssid_mapping_v1(
                    limit=limit,
                    offset=offset,
                    headers=headers,
                    **request_parameters
        )


    # Alias Function
    def update_sda_wireless_multicast(self,
                                         fabric_id,
                                         multicastEnabled=None,
                                         headers=None,
                                         payload=None,
                                         active_validation=True,
                                         **request_parameters):
        """ This function is an alias of update_sda_wireless_multicast_v1 .
        Args:
            multicastEnabled(boolean): Fabric Wireless's Multicast Enabled.
            fabric_id(str): fabricId path parameter. The unique identifier of the fabric site for which the
                multicast setting is being requested. The identifier should be in the format of a UUID.
                The 'fabricId' can be obtained using the api /dna/intent/api/v1/sda/fabricSites. .
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            payload(): A JSON serializable Python object to send in the
                body of the Request.
            active_validation(bool): Enable/Disable payload validation.
                Defaults to True.
            **request_parameters: Additional request parameters (provides
                support for parameters that may be added in the future).

        Returns:
            This function returns the output of update_sda_wireless_multicast_v1 .
        """
        return self.update_sda_wireless_multicast_v1(
                    fabric_id=fabric_id,
                    multicastEnabled=multicastEnabled,
                    headers=headers,
                    payload=payload,
                    active_validation=active_validation,
                    **request_parameters
        )


    # Alias Function
    def switch_wireless_setting_and_rolling_ap_upgrade_management(self,
                                                                     fabric_id,
                                                                     enableWireless=None,
                                                                     id=None,
                                                                     rollingApUpgrade=None,
                                                                     headers=None,
                                                                     payload=None,
                                                                     active_validation=True,
                                                                     **request_parameters):
        """ This function is an alias of switch_wireless_setting_and_rolling_ap_upgrade_management_v1 .
        Args:
            enableWireless(boolean): Fabric Wireless's Enable Wireless.
            id(string): Fabric Wireless's Network Device ID of the wireless capable switch .
            rollingApUpgrade(object): Fabric Wireless's rollingApUpgrade.
            fabric_id(str): fabricId path parameter. The 'fabricId' represents the Fabric ID of a particular
                Fabric Site. The 'fabricId' can be obtained using the api
                /dna/intent/api/v1/sda/fabricSites.  Example : e290f1ee-6c54-4b01-90e6-d701748f0851 .
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            payload(): A JSON serializable Python object to send in the
                body of the Request.
            active_validation(bool): Enable/Disable payload validation.
                Defaults to True.
            **request_parameters: Additional request parameters (provides
                support for parameters that may be added in the future).

        Returns:
            This function returns the output of switch_wireless_setting_and_rolling_ap_upgrade_management_v1 .
        """
        return self.switch_wireless_setting_and_rolling_ap_upgrade_management_v1(
                    fabric_id=fabric_id,
                    enableWireless=enableWireless,
                    id=id,
                    rollingApUpgrade=rollingApUpgrade,
                    headers=headers,
                    payload=payload,
                    active_validation=active_validation,
                    **request_parameters
        )


    # Alias Function
    def add_w_l_c_to_fabric_domain(self,
                                      deviceName=None,
                                      siteNameHierarchy=None,
                                      headers=None,
                                      payload=None,
                                      active_validation=True,
                                      **request_parameters):
        """ This function is an alias of add_w_l_c_to_fabric_domain_v1 .
        Args:
            deviceName(string): Fabric Wireless's WLC Device Name .
            siteNameHierarchy(string): Fabric Wireless's Fabric Site Name Hierarchy .
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            payload(): A JSON serializable Python object to send in the
                body of the Request.
            active_validation(bool): Enable/Disable payload validation.
                Defaults to True.
            **request_parameters: Additional request parameters (provides
                support for parameters that may be added in the future).

        Returns:
            This function returns the output of add_w_l_c_to_fabric_domain_v1 .
        """
        return self.add_w_l_c_to_fabric_domain_v1(
                    deviceName=deviceName,
                    siteNameHierarchy=siteNameHierarchy,
                    headers=headers,
                    payload=payload,
                    active_validation=active_validation,
                    **request_parameters
        )


    # Alias Function
    def add_ssid_to_ip_pool_mapping(self,
                                       scalableGroupName=None,
                                       siteNameHierarchy=None,
                                       ssidNames=None,
                                       vlanName=None,
                                       headers=None,
                                       payload=None,
                                       active_validation=True,
                                       **request_parameters):
        """ This function is an alias of add_ssid_to_ip_pool_mapping_v1 .
        Args:
            scalableGroupName(string): Fabric Wireless's Scalable Group Name .
            siteNameHierarchy(string): Fabric Wireless's Site Name Hierarchy .
            ssidNames(list): Fabric Wireless's List of SSIDs  (list of strings).
            vlanName(string): Fabric Wireless's VLAN Name .
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            payload(): A JSON serializable Python object to send in the
                body of the Request.
            active_validation(bool): Enable/Disable payload validation.
                Defaults to True.
            **request_parameters: Additional request parameters (provides
                support for parameters that may be added in the future).

        Returns:
            This function returns the output of add_ssid_to_ip_pool_mapping_v1 .
        """
        return self.add_ssid_to_ip_pool_mapping_v1(
                    scalableGroupName=scalableGroupName,
                    siteNameHierarchy=siteNameHierarchy,
                    ssidNames=ssidNames,
                    vlanName=vlanName,
                    headers=headers,
                    payload=payload,
                    active_validation=active_validation,
                    **request_parameters
        )


    # Alias Function
    def update_ssid_to_ip_pool_mapping(self,
                                          scalableGroupName=None,
                                          siteNameHierarchy=None,
                                          ssidNames=None,
                                          vlanName=None,
                                          headers=None,
                                          payload=None,
                                          active_validation=True,
                                          **request_parameters):
        """ This function is an alias of update_ssid_to_ip_pool_mapping_v1 .
        Args:
            scalableGroupName(string): Fabric Wireless's Scalable Group Name .
            siteNameHierarchy(string): Fabric Wireless's Site Name Hierarchy .
            ssidNames(list): Fabric Wireless's List of SSIDs  (list of strings).
            vlanName(string): Fabric Wireless's VLAN Name .
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            payload(): A JSON serializable Python object to send in the
                body of the Request.
            active_validation(bool): Enable/Disable payload validation.
                Defaults to True.
            **request_parameters: Additional request parameters (provides
                support for parameters that may be added in the future).

        Returns:
            This function returns the output of update_ssid_to_ip_pool_mapping_v1 .
        """
        return self.update_ssid_to_ip_pool_mapping_v1(
                    scalableGroupName=scalableGroupName,
                    siteNameHierarchy=siteNameHierarchy,
                    ssidNames=ssidNames,
                    vlanName=vlanName,
                    headers=headers,
                    payload=payload,
                    active_validation=active_validation,
                    **request_parameters
        )


    # Alias Function
    def returns_the_count_of_vlans_mapped_to_ssids_in_a_fabric_site(self,
                                                                       fabric_id,
                                                                       headers=None,
                                                                       **request_parameters):
        """ This function is an alias of returns_the_count_of_vlans_mapped_to_ssids_in_a_fabric_site_v1 .
        Args:
            fabric_id(str): fabricId path parameter. The 'fabricId' represents the Fabric ID of a particular
                Fabric Site .
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            **request_parameters: Additional request parameters (provides
                support for parameters that may be added in the future).

        Returns:
            This function returns the output of returns_the_count_of_vlans_mapped_to_ssids_in_a_fabric_site_v1 .
        """
        return self.returns_the_count_of_vlans_mapped_to_ssids_in_a_fabric_site_v1(
                    fabric_id=fabric_id,
                    headers=headers,
                    **request_parameters
        )


    # Alias Function
    def get_sda_wireless_multicast(self,
                                      fabric_id,
                                      headers=None,
                                      **request_parameters):
        """ This function is an alias of get_sda_wireless_multicast_v1 .
        Args:
            fabric_id(str): fabricId path parameter. The unique identifier of the fabric site for which the
                multicast setting is being requested. The identifier should be in the format of a UUID.
                The 'fabricId' can be obtained using the api /dna/intent/api/v1/sda/fabricSites. .
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            **request_parameters: Additional request parameters (provides
                support for parameters that may be added in the future).

        Returns:
            This function returns the output of get_sda_wireless_multicast_v1 .
        """
        return self.get_sda_wireless_multicast_v1(
                    fabric_id=fabric_id,
                    headers=headers,
                    **request_parameters
        )


    # Alias Function
    def return_the_count_of_all_the_fabric_site_which_has_ssid_to_ip_pool_mapping(self,
                                                                                     headers=None,
                                                                                     **request_parameters):
        """ This function is an alias of return_the_count_of_all_the_fabric_site_which_has_ssid_to_ip_pool_mapping_v1 .
        Args:
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            **request_parameters: Additional request parameters (provides
                support for parameters that may be added in the future).

        Returns:
            This function returns the output of return_the_count_of_all_the_fabric_site_which_has_ssid_to_ip_pool_mapping_v1 .
        """
        return self.return_the_count_of_all_the_fabric_site_which_has_ssid_to_ip_pool_mapping_v1(
                    headers=headers,
                    **request_parameters
        )


    # Alias Function
    def reload_switch_for_wireless_controller_cleanup(self,
                                                         fabric_id,
                                                         deviceId=None,
                                                         headers=None,
                                                         payload=None,
                                                         active_validation=True,
                                                         **request_parameters):
        """ This function is an alias of reload_switch_for_wireless_controller_cleanup_v1 .
        Args:
            deviceId(string): Fabric Wireless's Network Device ID .
            fabric_id(str): fabricId path parameter. The 'fabricId' represents the Fabric ID of a particular
                Fabric Site. The 'fabricId' can be obtained using the api
                /dna/intent/api/v1/sda/fabricSites.  Example : e290f1ee-6c54-4b01-90e6-d701748f0851 .
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            payload(): A JSON serializable Python object to send in the
                body of the Request.
            active_validation(bool): Enable/Disable payload validation.
                Defaults to True.
            **request_parameters: Additional request parameters (provides
                support for parameters that may be added in the future).

        Returns:
            This function returns the output of reload_switch_for_wireless_controller_cleanup_v1 .
        """
        return self.reload_switch_for_wireless_controller_cleanup_v1(
                    fabric_id=fabric_id,
                    deviceId=deviceId,
                    headers=headers,
                    payload=payload,
                    active_validation=active_validation,
                    **request_parameters
        )


    # Alias Function
    def get_ssid_to_ip_pool_mapping(self,
                                       site_name_hierarchy,
                                       vlan_name,
                                       headers=None,
                                       **request_parameters):
        """ This function is an alias of get_ssid_to_ip_pool_mapping_v1 .
        Args:
            vlan_name(str): vlanName query parameter. VLAN Name .
            site_name_hierarchy(str): siteNameHierarchy query parameter. Site Name Heirarchy .
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            **request_parameters: Additional request parameters (provides
                support for parameters that may be added in the future).

        Returns:
            This function returns the output of get_ssid_to_ip_pool_mapping_v1 .
        """
        return self.get_ssid_to_ip_pool_mapping_v1(
                    site_name_hierarchy=site_name_hierarchy,
                    vlan_name=vlan_name,
                    headers=headers,
                    **request_parameters
        )


    # Alias Function
    def get_sda_wireless_details_from_switches(self,
                                                  fabric_id,
                                                  headers=None,
                                                  **request_parameters):
        """ This function is an alias of get_sda_wireless_details_from_switches_v1 .
        Args:
            fabric_id(str): fabricId path parameter. The 'fabricId' represents the Fabric ID of a particular
                Fabric Site. The 'fabricId' can be obtained using the api
                /dna/intent/api/v1/sda/fabricSites.  Example : e290f1ee-6c54-4b01-90e6-d701748f0851 .
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            **request_parameters: Additional request parameters (provides
                support for parameters that may be added in the future).

        Returns:
            This function returns the output of get_sda_wireless_details_from_switches_v1 .
        """
        return self.get_sda_wireless_details_from_switches_v1(
                    fabric_id=fabric_id,
                    headers=headers,
                    **request_parameters
        )


    # Alias Function
    def retrieve_the_vlans_and_ssids_mapped_to_the_vlan_within_a_fabric_site(self,
                                                                                fabric_id,
                                                                                limit=None,
                                                                                offset=None,
                                                                                headers=None,
                                                                                **request_parameters):
        """ This function is an alias of retrieve_the_vlans_and_ssids_mapped_to_the_vlan_within_a_fabric_site_v1 .
        Args:
            fabric_id(str): fabricId path parameter. The 'fabricId' represents the Fabric ID of a particular
                Fabric Site .
            limit(int): limit query parameter. The number of records to show for this page. Default is 500 if not
                specified. Maximum allowed limit is 500. .
            offset(int): offset query parameter. The first record to show for this page; the first record is
                numbered 1. .
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            **request_parameters: Additional request parameters (provides
                support for parameters that may be added in the future).

        Returns:
            This function returns the output of retrieve_the_vlans_and_ssids_mapped_to_the_vlan_within_a_fabric_site_v1 .
        """
        return self.retrieve_the_vlans_and_ssids_mapped_to_the_vlan_within_a_fabric_site_v1(
                    fabric_id=fabric_id,
                    limit=limit,
                    offset=offset,
                    headers=headers,
                    **request_parameters
        )

    def add_update_or_remove_ssid_mapping_to_a_vlan(self,
                                                       fabric_id,
                                                       headers=None,
                                                       payload=None,
                                                       active_validation=True,
                                                       **request_parameters):
        """ This function is an alias of add_update_or_remove_ssid_mapping_to_a_vlan_v1 .
        Args:
            fabric_id(str): fabricId path parameter. The 'fabricId' represents the Fabric ID of a particular
                Fabric Site .
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            payload(list): A JSON serializable Python object to send in the
                body of the Request.
            active_validation(bool): Enable/Disable payload validation.
                Defaults to True.
            **request_parameters: Additional request parameters (provides
                support for parameters that may be added in the future).

        Returns:
            This function returns the output of add_update_or_remove_ssid_mapping_to_a_vlan_v1 .
        """
        return self.add_update_or_remove_ssid_mapping_to_a_vlan_v1(
                                                       fabric_id,
                                                       headers,
                                                       payload,
                                                       active_validation,
                                                       **request_parameters)


